import QtQuick 2.12
import QtQuick.Layouts 1.12
//import QtAV 1.6
import QtMultimedia 5.7
import Components 1.0
import FileIO 1.0
import StorageMgr 1.0

Item {
    id: root
    property string videoDirPath: "/data/video/attract/"
    property string videoFile: ""
//    property bool isVideoAvailable: videoArtwork.source !== ""
    property string imageFile: ""

    clip: true

    function stop() {
        videoArtwork.stopVideo()
    }

    function start() {
        if(videoArtwork.playbackState !== MediaPlayer.PlayingState) {
            if(videoArtwork.playbackState === MediaPlayer.PausedState) {
                videoArtwork.play()
            } else {
                videoArtwork.startVideo()
            }
        }
    }

    function pause() {
        videoArtwork.pause()
    }

    FileIO {
        id: fileio
    }

    StorageMgr {
        id: storageMgr
    }

    function getFirstVideoFile(path) {
        var files = storageMgr.listFiles(path, ["*.mpg", "*.mpeg", "*.avi", "*.mp4"])
        if(files.length > 0) {
            return path + "/" + files[0]
        } else {
            return ""
        }
    }
    function getFirstImageFile(path) {
        var files = storageMgr.listFiles(path, ["*.jpg", "*.jpeg", "*.png", "*.bmp"])
        if(files.length > 0) {
            return path + "/" + files[0]
        } else {
            return ""
        }
    }

    function checkForVideo() {
        var file = ""
        if(fileio.dirIsEmpty(videoDirPath + "user") === false) {
            file = getFirstVideoFile(videoDirPath + "user")
        } else if(fileio.dirIsEmpty(videoDirPath + "default") === false) {
            if(main.modestyMode === true) {
                file = "/data/video/attract/default/home_modesty.mp4"
            } else {
                file = "/data/video/attract/default/home.mp4"
            }
        }
        return file
    }

    Image {
        id: imageArtwork
        fillMode: Image.PreserveAspectFit
        source: ""
        anchors.fill: parent
        visible: source !== ""
        Component.onCompleted: source = getFirstImageFile(videoDirPath + "user")
    }

    Video {
        id: videoArtwork
        fillMode: VideoOutput.PreserveAspectCrop

        function startVideo() {
            imageArtwork.source = getFirstImageFile(videoDirPath + "user")
            videoArtwork.source = checkForVideo()
            videoArtwork.seek(1)
            videoArtwork.play()
        }

        function stopVideo() {
            videoArtwork.stop()
            videoArtwork.source = ""
        }

        anchors {
            left: root.left
            verticalCenter: root.verticalCenter
        }

        height: parent.height
        width: parent.width

        muted: true
//        loops: MediaPlayer.Infinite

        onPlaybackStateChanged: {
            if(videoArtwork.playbackState === MediaPlayer.PlayingState)
                console.log("Attract Video Playing " + videoArtwork.source)

//            if(videoArtwork.playbackState === MediaPlayer.Loaded) {
//                root.isVideoAvailable = true
//            }

            if(videoArtwork.playbackState == MediaPlayer.StoppedState || videoArtwork.playbackState == MediaPlayer.PausedState) {
                if(visible)
                    videoArtwork.play()
            }
        }

        onPositionChanged: {
            if(duration > 0 && position >= (duration-500)) {
                startVideo()
            }
        }

        Component.onCompleted: {
//            startVideo()
        }
    }
}
