import QtQuick 2.12
import "../qml/"

Rectangle {
    id: root

    property variant model
    property variant selectedData: model.get(currentIndex)
    property alias currentIndex: listView.currentIndex
    property int comboHeight: 50
    property int itemHeight: comboBox.height / 2
    property url dropDownIcon: "graphics/ComboBox/down_arrow.png"
    property color highlightColor: main.colorUIBorder
    property color comboColor: main.colorHighlightBackground
    property color itemColor: "lightgray"
    property bool listenKeys: false

    signal comboClicked()

    width: 400 * main.hScaleFactor
    height: 400 * main.vScaleFactor
    color: "transparent"

    Rectangle {
        id: comboBox
        width: parent.width
        height: root.comboHeight
        color: "transparent"
        z: 100
        smooth: true

        Rectangle {
            id: chosenItem
            width: parent.width
            height: parent.height
            color: root.comboColor
            smooth: true
            Text {
                anchors.centerIn: parent
                id: chosenItemText
                text: listView.currentItem.option.text
                font.pixelSize: 14 * main.vScaleFactor
                color: main.colorFontLight
                smooth: true
            }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    buzzer.buttonPress()
                    comboBox.state = comboBox.state === "DropDown" ? "" : "DropDown"
                }
                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }

        ScalableImage {
            id: dropDownArrow
            anchors {
                right: chosenItem.right
                rightMargin: 10
                verticalCenter: chosenItem.verticalCenter
            }
            sourceName: "Bullet_Triangle_Gray"
        }

        Rectangle {
            id: dropDown
            width: comboBox.width
            height: 0
            clip: true
            anchors.top: chosenItem.bottom
            anchors.margins: 2 * main.vScaleFactor
            color: root.itemColor

            ListView {
                id: listView
                height: 500 * main.vScaleFactor
                model: root.model
                currentIndex: 0
//                keyNavigationWraps: true
                highlight: highlight
                delegate: Item{
                    width: comboBox.width
                    height: itemHeight
                    property alias option: option

                    Text {
                        id: option
                        text: model.label
                        anchors.top: parent.top
                        anchors.left: parent.left
                        anchors.margins: 5
                        color: main.colorFontLight

                    }
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            buzzer.buttonPress()
                            comboBox.state = ""
                            if (index !== listView.currentIndex)
                                root.comboClicked()
                            listView.currentIndex = index
                        }
                        Rectangle {
                            anchors.fill: parent
                            color: main.colorGenericStimulusArea
                            opacity: 0.3
                            visible: main.showStimulusArea
                        }
                    }
                }
            }
        }

        Component {
            id: highlight
            Rectangle {
                width: comboBox.width
                height: root.itemHeight
                color: root.highlightColor
            }
        }

        states: State {
            name: "DropDown"
            PropertyChanges { target: dropDown; height:root.itemHeight * root.model.count + 1 }
        }

        transitions: Transition {
            NumberAnimation { target: dropDown; properties: "height"; easing.type: Easing.OutExpo; duration: 1000 }
        }
    }
}
