import QtQuick 2.12
//import Components 1.0

ModalDialog {
    id: root

    isTransparent: true

    signal acceptClicked()
    signal cancelClicked()

    property string iconSourceName: "BikeMode/Bike_Mode_Icon"
    property string promptText: qsTr("Bike Mode?") + application.emptyString
    property string promptDesc: qsTr("You will be changing resistance to constant torque which simulates a bicycle more closely.") + application.emptyString

    Item {
        z: root.z + 1
        anchors.fill: parent

        Text {
            id: promptTextLabel
            width: parent.width
            color: main.colorFontLight
            text: root.promptText
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: parent.top
            anchors.topMargin: 45 * main.vScaleFactor
            horizontalAlignment: Text.AlignHCenter
            wrapMode: Text.WordWrap
            font.bold: false
            font.pixelSize: 20 * main.vScaleFactor
        }

        Text {
            id: promptDescription
            width: parent.width
            color: main.colorFontLight
            text: root.promptDesc
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: promptTextLabel.bottom
            anchors.topMargin: 5 * main.vScaleFactor
            horizontalAlignment: Text.AlignHCenter
            wrapMode: Text.WordWrap
            font.bold: false
            font.pixelSize: 15 * main.vScaleFactor
        }

        StandardButton {
            id: cancelButton

            width: 50 * main.hScaleFactor
            height: 50 * main.vScaleFactor
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 22 * main.vScaleFactor
            anchors.left: parent.left
            anchors.leftMargin: 80 * main.hScaleFactor

            icon.anchors.topMargin: 5 * main.vScaleFactor
            icon.anchors.bottomMargin: 5 * main.vScaleFactor

            icon.sourceName: "Components/tools_close";
            onClicked: cancelClicked()
        }

        StandardButton {
            id: acceptButton

            anchors.top: promptDescription.bottom
            anchors.topMargin: 22 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            buttonText: qsTr("CONFIRM") + application.emptyString

            icon.sourceName: root.iconSourceName

            onClicked: acceptClicked()
        }

        TextLabel {
            id: cancelText

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: countdownText.top
            text: qsTr("CANCEL IN:") + application.emptyString
            font.pixelSize: 12 * main.vScaleFactor
        }

        Text { // show count down to timeout of prompt (started by startTimeout(seconds) method of ModalDialog superclass
            id: countdownText

            color: main.colorFontLight

            // display count down (with a leading 0 if remaining seconds < 10
            text: ":" + ((countDown.toString().length < 2) ? "0" : "") + countDown
            font.bold: false
            font.pixelSize: 50 * main.vScaleFactor

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 10 * main.vScaleFactor
        }
    }
}
