import QtQuick 2.12
import "Globals.js" as Globals

Item {
    id: root

    // default time as seconds since unix epoch
    property int currentTime: Math.round(new Date().getTime() / 1000)
    property int time: computeTimestamp()
    property int curIndex: 0
    property int controlWidth: 0.15*width

    signal exitedList()

    width: 0.95 * parent.width
    height: 120 * main.vScaleFactor

    function getField(name) {
        switch (name) {
            case "MONTH":
                return monthPicker
            case "DAY":
                return dayPicker
            case "YEAR":
                return yearPicker
            case "HOURS":
                return hoursPicker
            case "MINUTES":
                return minutesPicker
            case "AM_PM":
                return amPmButton
        }
    }

    function getMonthName(index) {
        return [qsTr("Jan"), qsTr("Feb"), qsTr("Mar"),
                qsTr("Apr"), qsTr("May"), qsTr("Jun"),
                qsTr("Jul"), qsTr("Aug"), qsTr("Sep"),
                qsTr("Oct"), qsTr("Nov"), qsTr("Dec")][index]
    }

    // return format is "09 Jul 2010 14:22:00" (RFC 2822)
    function getDateString() {
        var year = yearPicker.value
        var month = monthPicker.value
        var day = dayPicker.value
        var hours = (hoursPicker.value % 12) + (amPmButton.value ? 12 : 0)
        var minutes = minutesPicker.value
        var timestring = Globals.padZeros(day, 2) + " "
                + getMonthName(month - 1) + " "
                + Globals.padZeros(year, 4) + " "
                + Globals.padZeros(hours,2) + ":"
                + Globals.padZeros(minutes,2)
                + ":00"
        return timestring
    }

    function computeTimestamp() {
        var timestring = getDateString()
        var ms = Date.parse(timestring)
        return Math.round(ms / 1000)
    }

    Component.onCompleted: {
        var date = new Date(currentTime * 1000)
        monthPicker.value = date.getMonth() + 1
        dayPicker.value = date.getDate()
        yearPicker.value = date.getFullYear()
        var hours = date.getHours()
        if (hours >= 12) {
            hoursPicker.value = hours === 12 ? 12 : hours % 12
            amPmButton.value = true
        } else {
            hoursPicker.value = hours === 0 ? 12 : hours
            amPmButton.value = false
        }
        minutesPicker.value = date.getMinutes()
    }

    Row {
        id: dateTimeRow
        width: parent.width
        height: parent.height
        spacing: 0

        NumberPicker {
            id: monthPicker
            width: controlWidth
            height: dateTimeRow.height
            minValue: 1
            maxValue: 12
            valueWraps: true
            onValueChanged: {
                var daysPerMonth = [31,28,31,30,31,30,31,31,30,31,30,31]
                if (value >= 1 && value <= 12) {
                    dayPicker.maxValue = daysPerMonth[value - 1]
                }
            }
        }

        Text {
            text: "/"
            width: 0.01 * root.width
            anchors.verticalCenter: parent.verticalCenter
            verticalAlignment: Text.AlignVCenter
            font.pixelSize: 30 * main.vScaleFactor
        }


        NumberPicker {
            id: dayPicker
            width: controlWidth
            height: dateTimeRow.height
            minValue: 1
            maxValue: 31
            valueWraps: true
        }

        Text {
            text: "/"
            width: 0.01 * root.width
            anchors.verticalCenter: parent.verticalCenter
            verticalAlignment: Text.AlignVCenter
            font.pixelSize: 30 * main.vScaleFactor
        }


        NumberPicker {
            id: yearPicker
            width: 1.0*controlWidth
            height: dateTimeRow.height
            minValue: 1
            maxValue: 3000
            defaultValue: 2013
            valueWraps: true
        }

        Item {
            // blank space
            width: 0.02 * root.width
            height: root.height
        }

        NumberPicker {
            id: hoursPicker
            width: controlWidth
            height: dateTimeRow.height
            minValue: 1
            maxValue: 12
            valueWraps: true
        }

        Text {
            text: ":"
            width: 0.01 * root.width
            anchors.verticalCenter: parent.verticalCenter
            verticalAlignment: Text.AlignVCenter
            font.pixelSize: 30 * main.vScaleFactor
        }

        NumberPicker {
            id: minutesPicker
            width: controlWidth
            height: dateTimeRow.height
            minValue: 0
            maxValue: 59
            valueWraps: true
            zeroPadding: 2
        }

        ToggleButton {
            id: amPmButton
            width: 0.2 * root.width
            height: 30 * main.vScaleFactor

            pos0label: qsTr("AM") + application.emptyString
            pos1label: qsTr("PM") + application.emptyString
            posFont.pixelSize: 20* main.vScaleFactor
            labelcolor: main.colorDiagnosticEdges
            highlightColor: main.colorDiagnosticEdges
            anchors.verticalCenter: parent.verticalCenter
        }
    }
}
