import QtQuick 2.12
//~ import QtQuick.Controls 2.12

Rectangle {
    id: expbox

    color: main.colorButtonBackground
    border.color: main.colorUIBorder
    border.width: main.widthBorder
    radius: main.defaultRadius
    
    /**
     * Begin collapsed (default state)
     */
    property bool expanded: false
    state: expanded ? "expanded" : ""
    
    /**
     * Flag to determine if clicking causes expanding
     */
    property bool clickToExpand: true
    
    /**
     * Spacing placed between two loaders (assuming expandedContent has dimensions)
     */
    property real spacing: 0
    
    /**
     * Time in ms for animations
     */
    property int  expandTime: 1000
    
    /**
     * Dimensions of box when collapsed and expanded
     */
    readonly property alias collapsedWidth:  collapsedContent.width
    readonly property alias collapsedHeight: collapsedContent.height
    readonly property real  expandedWidth:   (expandedContent.width > 0) ? expandedContent.width : collapsedWidth
    readonly property real  expandedHeight:  (expandedContent.height > 0) ? collapsedHeight+spacing+expandedContent.height : collapsedHeight
    
    onCollapsedWidthChanged: width = collapsedWidth
    onCollapsedHeightChanged: height = collapsedHeight
    
    /**
     * Loader to hold content shown when collapsed (and also expanded)
     */
    property alias collapsedLoader: collapsedContent
    
    /**
     * Loader to hold content shown only when expanding
     */
    property alias extensionLoader: expandedContent

    /**
     * Boolean to determine if the stimulus area should be shown.
     */
    property bool isClickable: true
    
    /**
     * 
     */
    signal clicked();
    
    onExpandedChanged: { state = expanded ? "expanded" : "collapsed" }
    
    /**
     * Simply have two states: collapsed and expanded
     * Use the appropriate height for each state
     */
    states: [
        State {
            name: ""
            PropertyChanges {
                target: expbox
                height: collapsedHeight
            }
        },
        State {
            name: "expanded"
            PropertyChanges {
                target: expbox
                height: expandedHeight
            }
        }
    ]
    /**
     * Animations when changing states
     */
    transitions: [
        Transition {
            ParallelAnimation {
                PropertyAnimation {
                    target: expbox
                    property: "height"
                    duration: expandTime
                }
            }
        }
    ]
    
    /**
     * Returns true when state is expanded, otherwise false
     */
    function isExpanded() {
        return expanded;
    }
    /**
     * Sets state to collapse the box
     */
    function collapse() {
        expanded = false;
    }
    /**
     * Sets state to expand the box
     */
    function expand() {
        expanded = true;
    }
    /**
     * Toggles the expanded state
     */
    function toggle() {
        expanded = !expanded
    }
    
    MouseArea {
        anchors.fill: parent
        enabled: isClickable
        onClicked: {
            if(clickToExpand === true) {
                expanded = !expanded
            }
            expbox.clicked();
            buzzer.buttonPress()
        }
        Rectangle {
            anchors.fill: parent
            color: main.colorGenericStimulusArea
            opacity: 0.3
            visible: main.showStimulusArea && isClickable
        }
    }
    
    Column {
        anchors.fill: parent
        spacing: (expandedContent.height > 0) ? expbox.spacing : 0
        clip: true
        
        Loader {
            id: collapsedContent
            asynchronous: true
            onSourceComponentChanged: {
                height = sourceComponent.height
                width = sourceComponent.width
            }
        }
        
        Loader {
            id: expandedContent
            width: parent.width // Match width of collapsedContent
            asynchronous: true
            clip: true
            onSourceComponentChanged: {
                height = sourceComponent.height
            }
        }
    }
}
