import QtQuick 2.12
import "../qml/"

Image {
    id: root
    
    property bool useSVG: main.useSVG
//    property bool useHighRes: main.useHighResPNG

    Component.onCompleted: {
        sourceFlags.completed = true; // Mark that the initial load has completed
        root.update();
    }
    
    source: ""
    property string svgSourceBase: "../qml/graphics/svg/"  // Relative base path for all images (Default only works for qml files in the qml or Components directory)
    property string defaultSourceBase: "../qml/graphics/ui3_0/"  // Relative base path for all images (Default only works for qml files in the qml or Components directory)
    property string sourceName: ""                 // Unique path/filename for image without a suffix
    property url sourceURL: useSVG ? svgSourceBase : defaultSourceBase
    readonly property string pngSuffix: ".png"
    readonly property string svgSuffix: ".svg"
    property string usedSuffix: ""
//    property url  sourcePNG        // Low-res PNG
//    property url  sourcePNGHighRes // High-res PNG
//    property url  sourceSVG        // SVG

    // If either flag changed and source was not explicitly set,
    //  re-evaluate to determine which source to use.
    onUseSVGChanged: root.update();
//    onUseHighResChanged: root.update();
    // If one of the sources changed and that source is currently displayed,
    //  update the source.  Reset the failed flag.
    onDefaultSourceBaseChanged: { sourceFlags.svgFailed = false; root.update(); }
    onSourceNameChanged: { sourceFlags.svgFailed = false; root.update(); }
//    onSourcePNGHighResChanged: { sourceFlags.pngHRFailed = false; root.update(); }
//    onSourceSVGChanged: { sourceFlags.svgFailed = false; root.update(); }

    // Private flags related to the loading of sources
    QtObject {
        id: sourceFlags
        
        // Flag set to true in Component.onCompleted
        //  Prevents update() from being called multiple times when setting sources
        //  before the image is even ready
        property bool completed: false;
        // Flag set to true if sourceSVG failed to load.  Prevents it from
        //  being loaded again.  Will fallback to one of PNGs.
        property bool svgFailed: false
        // Flag set to true if sourcePNGHighRes failed to load.  Prevents
        //  it from being loaded again.  Will fallback to low-res PNG.
//        property bool pngHRFailed: false
    }
    
    function update() {
        // Only update the source if the initial load has completed
        if(sourceFlags.completed && root.sourceName != undefined && root.sourceName != "") {
            if(root.useSVG && !sourceFlags.svgFailed) {
                root.usedSuffix = root.svgSuffix
            }
            else {
                root.usedSuffix = root.pngSuffix
            }
            root.source = root.sourceURL + root.sourceName + root.usedSuffix
//            if(root.useSVG && !sourceFlags.svgFailed && root.sourceSVG != undefined && root.sourcePNG != "") {
//                root.source = root.sourceSVG;
//            } else if(root.useHighRes && !sourceFlags.pngHRFailed && root.sourcePNGHighRes != undefined && root.sourcePNG != "") {
//                root.source = root.sourcePNGHighRes;
//            } else if(root.sourcePNG != undefined && root.sourcePNG != "") {
//                root.source = root.sourcePNG;
//        }
        }
    }
    
    /**
     * Prefixes and suffixes used by getXFilename()
     *   Allows for streamlined setting of sources.
     *   Prefix will typically be the directory
     *   Suffix will typically be the extension
     *   
     * For example:
     *   To separate each set of images into their own directory,
     *    use something like the following:
     *   
     *    pngPrefix: "graphics/default/"
     *    pngSuffix: ".png"
     *    pngHighResPrefix: "graphics/highres/"
     *    pngHighResSuffix: ".png"
     *    svgPrefix: "graphics/svg/"
     *    svgSuffix: ".svg"
     * 
     *   Then when instantiating ScalableImage, set the following properties:
     *    sourcePNG: getPNGFilename("image_01") // "graphics/default/image_01.png"
     *    sourcePNGHighRes: getPNGHighResFilename("image_01") // "graphics/highres/image_01.png"
     *    sourceSVG: getSVGFilename("image_01") // "graphics/svg/image_01.svg"
     * 
     *   
     */
//    property string pngPrefix: "graphics/"
//    readonly property string pngSuffix: ".png"
//    property string pngHighResPrefix: "graphics/"
//    readonly property string pngHighResSuffix: "_HighRes.png"
//    property string svgPrefix: "graphics/"
//    readonly property string svgSuffix: "_svg.png" // FOR TESTING *********************************************************************************************************

    /**
     * These functions are used to generate the appropriate filename for each image.
     *   Call these when instantiating a ScalableImage, so the URL is taken from the
     *   directory that the ScalableImage is being instantiated in, rather than the
     *   Components/ directory (where this file is located).
     */
//    function getPNGFilename(base) {
//        return pngPrefix+base+pngSuffix;
//    }

//    function getPNGHighResFilename(base) {
//        return pngHighResPrefix+base+pngHighResSuffix;
//    }

//    function getSVGFilename(base) {
//        return svgPrefix+base+svgSuffix;
//    }

    onStatusChanged: {
        if(status == Image.Error) {
            if(root.usedSuffix === root.svgSuffix) {
                sourceFlags.svgFailed = true;
                root.update();
            } else if(root.usedSuffix === root.pngSuffix) {
//                sourceFlags.pngFailed = true;
                root.update();
            } else {
                console.log("ScalableImage: Could not load any image");
            }
        }
    }
}
