import QtQuick 2.12

/**
 * @brief Image that is the basis of a image only button within the application.
 */ 
ScalableImage {
    id: icon
    fillMode: Image.PreserveAspectFit
    smooth: true

    /**
     * @brief Signal for when a mouse click on the image has started.
     */
    signal pressed();

    /**
     * @brief Signal for when a mouse click on the image has been released.
     */
    signal released();

    /**
     * @brief Signal for when the image has been clicked.
     */
    signal clicked();

    /**
     * @brief Signal for when the image has been a long press.
     */
    signal pressAndHold();

    Rectangle {
        anchors.fill: parent
        anchors.margins: -5
        color: main.colorGenericStimulusArea
        opacity: 0.3
        visible: main.showStimulusArea
    }

    MouseArea {
        anchors.fill: parent
        onPressAndHold: {
            icon.pressAndHold();
            buzzer.buttonPress()
        }
        onPressed: icon.pressed();
        onReleased: icon.released();
        onClicked: {
            icon.clicked();
            buzzer.buttonPress()
        }
    }
}
