import QtQuick 2.12

/**
 * @brief Rectangle that is the basis of standarized keyboard/keypad buttons within the application.
 */
Rectangle {
    id: standardKeyboardButton
    width: singleWidth
    height: singleHeight
    radius: main.defaultRadius
    border.color: main.colorUIBorder
    border.width: main.widthBorder
    color: selected ? main.colorSelected : backgroundColor
    state: "single"
    
    property int singleWidth: 66 * main.hScaleFactor
    property int singleHeight: 73 * main.vScaleFactor
    property alias icon: b_image
    property bool isNumberLabel: false

    /**
     * @brief type:string The text displayed within the keyboard button.
     */
    property string buttonText: ""

    /**
     * @brief type:color The color of the text within the keyboard button.
     */
    property color textColor: "white"
    /**
     * @brief type:color The background color to use for the keyboard button.
     */
    property color backgroundColor: "black"

    /**
     * @brief type:int The size of the text within the keyboard button.
     */
    property int fontsize: 45

    /**
     * @brief type:bool Determines if the keyboard button is in a selected button state.
     */
    property bool selected: false

    property int textTopMargin: 3

    states: [
        State {
            name: "single"
            PropertyChanges { target: standardKeyboardButton ; width: singleWidth }
        },
        State {
            name: "double"
            PropertyChanges { target: standardKeyboardButton ; width: 137 * main.hScaleFactor }
        },
        State {
            name: "triple"
            PropertyChanges { target: standardKeyboardButton ; width: 208 * main.hScaleFactor }
        },
        State {
            name: "close"
            PropertyChanges { target: standardKeyboardButton ; width: 30 * main.hScaleFactor ; height: 30 * main.vScaleFactor ; fontsize: 20 ; buttonText: "X" }
        }
    ]

    /**
     * @brief Signal for when the keyboard button has been clicked.
     */
    signal clicked()

    Text {
        id: b_text
        anchors {
            top: parent.top
            topMargin: textTopMargin * main.vScaleFactor
            horizontalCenter: parent.horizontalCenter
        }
        text: standardKeyboardButton.buttonText
        horizontalAlignment: Text.AlignHCenter
        font.pixelSize: fontsize * main.vScaleFactor
        font.family: isNumberLabel ? "Free Sans" : "Montserrat"
        font.bold: isNumberLabel ? true : false
        color: standardKeyboardButton.textColor
    }
    
    ScalableImage {
        id: b_image
        anchors {
            verticalCenter: parent.verticalCenter
            horizontalCenter: parent.horizontalCenter
        }
        width: sourceSize.width * main.hScaleFactor
        height: sourceSize.height * main.vScaleFactor
        fillMode: Image.Stretch
        onStatusChanged: {
            if(b_image.status === Image.Error) {
                b_image.enabled = false;
                b_image.visible = false;
            } else {
                b_text.visible = false;
            }
        }
    }

    MouseArea {
        anchors.fill: parent
        onClicked:  {
            standardKeyboardButton.selected = false;
            standardKeyboardButton.clicked();
            buzzer.buttonPress()
        }
        onPressed: standardKeyboardButton.selected = true;
        onReleased: standardKeyboardButton.selected = false;

        Rectangle {
            anchors.fill: parent
            color: main.colorGenericStimulusArea
            opacity: 0.3
            visible: main.showStimulusArea
        }
    }


}
