import QtQuick 2.12
import Process 0.1
import FileIO 1.0
import "../string_format.js" as StrFormat


Item {

    id: root
    property bool dummy: true
    property bool debug: true
    property bool advertising: false


    //signal advertise()
    function debugPrint(string) {
        if(root.debug)
            print(string)
    }


    function  startAdvertiseing() {
        console.log("Bluetooth LE: Start advertising")
        var args=["start"]
        if(root.debug) {
            args= args.concat(["-d"])

        }
        bleno.arguments = args
        if(root.debug) print("Bluetooth LE: bleno arguments = " + bleno.arguments)

        bleno.start()
    }

    function stopAdvertising(){
        console.log("Bluetooth LE: Stop advertising")
        bleno.terminate()
        killDaemons.start() // This will call the shell script again with the "stop" command to kill the child processes as well
        //killDaemons.terminate()
    }

    Process {
        id: killDaemons
        program: "scripts/killBlenoDaemons.sh"
        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            debugPrint( "killDaemons process (STDOUT): " + output)
        }
    }


    Process{
        id: bleno

        program: "scripts/startBleno.sh"
       
        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            debugPrint( "bleno process (STDOUT): " + output)
        }

        onReadyReadStandardError: {
            var output = String(readAllStandardError())
            debugPrint( "bleno process (STDERR): " + output)
        }
    }



    FileIO {
        id:fileio
    }


    function exportWorkoutDataForBleno(summary,application) {

        // Export workoutData
        //Import template file

        fileio.source = "qml/BlueTooth/bleno/trueTemplates/blenoWorkoutServiceTEMPLATE.js"
        fileio.fileMode = FileIO.READ_ONLY
        fileio.open()
        fileio.clear()
        var template =new String()
        var charType

        while(!fileio.atEnd()){
            template = template.concat(fileio.readLine())
        }
        fileio.close()

        // Insert data into template
        fileio.clear()
        fileio.source = "qml/BlueTooth/bleno/trueScripts/blenoWorkoutStatsService.js"
        fileio.fileMode = FileIO.READ_WRITE
        fileio.open()
        fileio.clear()

        var line = new Array();
        var dynamicVal =new Array();
        line=template.split("\n")
        for (var i=0; i< line.length; i++ ){

            if(line[i].match("REPLACE")){
                charType=line[i].split(" ")[2]
                console.log("Found REPLACE line " + charType)
                switch(charType){
                case "ElapsedTimeCharacteristic":
                    dynamicVal= StrFormat.formatTimeStringMinAndSec(summary.totalTime)
                    break
                case "CaloriesCharacteristic":
                    dynamicVal=summary.totalCalories
                    break
                case "AvgHeartRateCharacteristic":
                    dynamicVal= summary.totalAverageHeartRate
                    break
                case "MaxHeartRateCharacteristic":
                    dynamicVal=summary.totalMaximumHeartRate
                    break
                case "AvgSpeedCharacteristic":
                    dynamicVal=summary.totalAverageSpeed
                    break
                case   "MaxSpeedCharacteristic":
                    dynamicVal= summary.totalMaximumSpeed
                    break
                case    "AvgPaceCharacteristic":
                    dynamicVal= summary.totalAveragePace
                    break
                case    "MaxPaceCharacteristic":
                    dynamicVal=summary.totalMaximumPace
                    break
                case    "TotalDistance":
                    dynamicVal= summary.totalDistance
                    break
                case    "FitnessScore":
                    dynamicVal= summary.fitTestScore
                    break
                case    "EquipmentType":
                    dynamicVal= summary.productTypeString
                    break

                }
                if(!dynamicVal) {
                    dynamicVal=0}
                line[i]=("   value: new Buffer('" + dynamicVal +"')," )

            }

            fileio.writeLine([line[i] + "\n"])
        }
        fileio.close()


        dynamicVal=0
        template=""
        line=[]
        // Export shortName for labeling
        // Export workoutData
        //Import template file

        fileio.source = "qml/BlueTooth/bleno/trueTemplates/advertiseWorkoutTEMPLATE.js"
        fileio.fileMode = FileIO.READ_ONLY
        fileio.open()
        // var template =new String()
        // var charType

        while(!fileio.atEnd()){
            template = template.concat(fileio.readLine())
        }
        fileio.close()

        // Insert data into template
        fileio.clear()
        fileio.source = "qml/BlueTooth/bleno/trueScripts/advertiseWorkout.js"
        fileio.fileMode = FileIO.READ_WRITE
        fileio.open()
        fileio.clear()
        //var line = new Array();
        ///var dynamicVal =new Array();
        line=template.split("\n")
        for ( i=0; i< line.length; i++ ){

            if(line[i].match("REPLACE")){
                charType=line[i].split(" ")[2]
                switch(charType){
                case "ShortName":
                    dynamicVal= StrFormat.formatTimeStringMinAndSec(summary.totalTime)
                    break
                }
                if(!dynamicVal) {
                    dynamicVal=0}
                line[i]=("    bleno.startAdvertising('Workout " + dynamicVal + "', ['9dc17002-8855-11e3-979a-78acc0b87777']);")

            }

            fileio.writeLine([line[i] + "\n"])
        }
        fileio.close()

// Export Device Information Service Data
        // Export workoutData
        //Import template file

        fileio.source = "qml/BlueTooth/bleno/trueTemplates/blenoDeviceInformationServiceTEMPLATE.js"
        fileio.fileMode = FileIO.READ_ONLY
        fileio.open()
        fileio.clear()
        var template =new String()
        var charType

        while(!fileio.atEnd()){
            template = template.concat(fileio.readLine())
        }
        fileio.close()

        // Insert data into template
        fileio.clear()
        fileio.source = "qml/BlueTooth/bleno/trueScripts/blenoDeviceInformationService.js"
        fileio.fileMode = FileIO.READ_WRITE
        fileio.open()
        fileio.clear()

        var line = new Array();
        var dynamicVal =new Array();
        line=template.split("\n")
        for (var i=0; i< line.length; i++ ){

            if(line[i].match("REPLACE")){
                charType=line[i].split(" ")[2]
                switch(charType){
                case "ModelNumberCharacteristic":
                    dynamicVal=machineinfo.modelString
                    break
                case "SerialNumberCharacteristic":
                    dynamicVal= application.serialNumber
                    break
                case "HardwareRevisionCharacteristic":
                    dynamicVal=machineinfo.brainboardFirmwareVersion
                    break
                case "SoftwareRevisionCharacteristic":
                    dynamicVal= application.appVersion
                    break
                }
                if(!dynamicVal) {
                    dynamicVal=0}
                line[i]=("   value: new Buffer('" + dynamicVal + "')," )

            }

            fileio.writeLine([line[i] + "\n"])
        }
        fileio.close()

    }



}
