import QtQuick 2.12
import Components 1.0
import ".."

ModalDialog {
    id: root
    dismissable: true

    height: 2 * main.heightDialog
    opacity: 1
    property string networkId: networkIdPicker.lumpValue()
    property string frequency: frequencyPicker.lumpValue()
    property string modulation: modulationForWscan(modPicker.lumpValue())
    property string symbolRate: symbolratePicker.lumpValue()

    function modulationForWscan() {
        // Translate into a form to be used by wscan "initial tuning data"
        switch (modPicker.lumpValue()) {
            case "64 QAM":
                return "QAM64"
            case "128 QAM":
                return "QAM128"
            case "256 QAM":
                return "QAM256"
        }
        return "QAMUNKNOWN"
    }

    onDismissed: {
        root.hide()
    }

    Item {
        z: root.z + 1
        anchors.fill: parent

        StandardKeyboardButton{
        id: close_button2

        z: 200
        anchors {   right: parent.right ; rightMargin: 100 * main.hScaleFactor ;
            top: parent.top ; topMargin: 40 * main.vScaleFactor
        }
        onClicked: root.dismissed()
        state:"close"
        }

        TextLabel {
            id: scanHeader
            width: 501 * main.hScaleFactor
            height: 99 * main.vScaleFactor
            text: qsTr("Network Scan") + application.emptyString
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: parent.top
            anchors.topMargin: 35 * main.vScaleFactor
            horizontalAlignment: Text.AlignHCenter
            wrapMode: Text.WordWrap
            font.pixelSize: 40 * main.vScaleFactor
        }

        Grid {
            id: controlList

            columnSpacing: 10 * main.vScaleFactor
            rowSpacing: 10 * main.hScaleFactor
            columns: 3
            rows: 4
            horizontalItemAlignment: Grid.AlignHCenter
            verticalItemAlignment: Grid.AlignVCenter

            property int labelWidth: 100 * main.hScaleFactor
            property int valueWidth: 50  * main.hScaleFactor
            property int unitsWidth: 50 * main.hScaleFactor
            property int rowHeight: 110 * main.vScaleFactor


            width: labelWidth + 6*valueWidth + unitsWidth + 2 * columnSpacing
            anchors {
                top: scanHeader.bottom
                topMargin: -20 * main.vScaleFactor
                bottom: okButton.top
                bottomMargin: 20 * main.vScaleFactor
                horizontalCenter: parent.horizontalCenter
            }

            Text {
                text: qsTr("Network ID") + application.emptyString
                width: controlList.labelWidth
                height:controlList. rowHeight
                color: main.colorFontLight
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignRight
            }

            ValuePickerRow {
                id: networkIdPicker
                numberOfItems: 5
                height: controlList.rowHeight
                width: numberOfItems * controlList.valueWidth
                delegate: Component {
                    NumberPicker {
                        id: picker
                        minValue: 0
                        maxValue: 9
                        defaultValue: 0
                        valueWraps: true
                        columnSpacing: 5 * main.vScaleFactor
                        width: controlList.valueWidth
                        height:90 * main.vScaleFactor
                        labelColor: main.colorFontLight
                    }
                }
            }

            Item {
                // Spacer
                height: controlList.rowHeight
                width: controlList.unitsWidth
            }

            Text {
                text: qsTr("Frequency") + application.emptyString
                width: controlList.labelWidth
                height: controlList.rowHeight
                color: main.colorFontLight
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignRight
            }

            ValuePickerRow {
                id: frequencyPicker
                numberOfItems: 6
                height: controlList.rowHeight
                width: numberOfItems * controlList.valueWidth
                delegate: Component {
                    NumberPicker {
                        id: picker
                        minValue: 0
                        maxValue: 9
                        defaultValue: 0
                        valueWraps: true
                        columnSpacing: 5 * main.hScaleFactor
                        width: controlList.valueWidth
                        height:90 * main.vScaleFactor
                        labelColor: main.colorFontLight
                    }
                }
            }

            Text {
                text: "KHz"
                width: controlList.unitsWidth
                height: controlList.rowHeight
                color: main.colorFontLight
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignLeft
            }

            Text {
                text: qsTr("Modulation") + application.emptyString
                width: controlList.labelWidth
                height: controlList.rowHeight
                color: main.colorFontLight
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignRight
            }

            ValuePickerRow {
                id: modPicker
                numberOfItems: 1
                height: controlList.rowHeight
                width: numberOfItems * 2 * controlList.valueWidth
                delegate: Component {
                    NumberPicker {
                        id: picker
                        ListModel {
                            id: qamModel
                            ListElement { label: "64 QAM"; value: 64 }
                            ListElement { label: "128 QAM"; value: 128 }
                            ListElement { label: "256 QAM"; value: 256 }
                        }
                        minValue: 0
                        maxValue: qamModel.count - 1
                        printedValue: qamModel.get(value).label
                        defaultValue: 0
                        valueWraps: true
                        columnSpacing: 5 * main.hScaleFactor
                        width: 2 * controlList.valueWidth
                        height:90 * main.vScaleFactor
                        labelColor: main.colorFontLight
                    }
                }
            }

            Item {
                // Spacer
                height: controlList.rowHeight
                width: controlList.unitsWidth
            }

            Text {
                text: qsTr("Symbol Rate") + application.emptyString
                width: controlList.labelWidth
                height: controlList.rowHeight
                color: main.colorFontLight
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignRight
            }

            ValuePickerRow {
                id: symbolratePicker
                numberOfItems: 4
                height: controlList.rowHeight
                width: numberOfItems * controlList.valueWidth
                delegate: Component {
                    NumberPicker {
                        id: picker
                        minValue: 0
                        maxValue: 9
                        defaultValue: 0
                        valueWraps: true
                        columnSpacing: 5 * main.vScaleFactor
                        width: controlList.valueWidth
                        height:90 * main.vScaleFactor
                        labelColor: main.colorFontLight
                    }
                }
            }

            Text {
                text: "KS/s"
                width: controlList.unitsWidth
                height: controlList.rowHeight
                color: main.colorFontLight
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignLeft
            }
        }

        StandardButton {
            id: okButton

            enabled: true
            buttonText: qsTr("OK") + application.emptyString
            height: 71 * main.vScaleFactor

            anchors {
                bottom: parent.bottom
                bottomMargin: 20 * main.vScaleFactor
                horizontalCenter: parent.horizontalCenter
            }

            onClicked: {
                root.dismissed()
            }
        }
    }
    Component.onCompleted: hide()
}
