import QtQuick 2.12
import Components 1.0
import "../Television"
import "../Keypad"

Rectangle {
    id: root

    color: main.colorToolbarBackground

    property bool isSelectable: false
    property int selectedIndex: -1
    property alias channelListGrid: channelListGrid
    property bool textActive: true

    property bool multipleSelectEnabled: false

    signal updatePreview()

   // onMultipleSelectEnabledChanged: tv.channelList.resetSelections()
    onMultipleSelectEnabledChanged: tv.channelList.resetSelections()

    Rectangle {
        id: sliderBar

        anchors.right: parent.right
        anchors.rightMargin: 2 * main.hScaleFactor
        width: 15 * main.hScaleFactor
        height: channelListGrid.height * channelListGrid.height/channelListGrid.contentHeight
        visible: channelListGrid.height < channelListGrid.contentHeight
        color: "darkgrey"
        radius: main.defaultRadius
        y: (parent.height - height) * channelListGrid.contentY / (channelListGrid.contentHeight - channelListGrid.height)
    }

    GridView {
        id: channelListGrid

        property int imageClickIndex

        onCurrentIndexChanged: {
            root.selectedIndex = currentIndex
            root.updatePreview()
            if(root.selectedIndex >= 0)
                currentItem.setTextFocus()
            else
                textActive = false
        }

        clip:true
        anchors.fill: parent
        anchors {
            leftMargin: 10 * main.hScaleFactor
            topMargin: 10 * main.vScaleFactor
            bottomMargin: 5 * main.vScaleFactor
            rightMargin: 5 * main.hScaleFactor
        }
        cellWidth: 130 * main.hScaleFactor
        cellHeight: 160 * main.vScaleFactor
       // model: tv.channelList.channels
         model: tv.channelList.channels

        delegate: Component {
            Item {
                function showIconSelector(){
                    if(root.isSelectable) {
                        channelListGrid.imageClickIndex = index
//                        iconSelector.show()
                    }
                }

                function setTextFocus() {
                    if(channelListGrid.currentIndex === index && root.textActive)
                        channelLabelText.forceActiveFocus()
                }

                width: 110 * main.hScaleFactor
                height: 110 * main.vScaleFactor

                ScalableImage {
                    id: highlightImage
                    visible: root.isSelectable && channelListGrid.currentIndex >= 0 && (root.multipleSelectEnabled ? selected : channelListGrid.currentIndex===index)
                    anchors.horizontalCenter: iconBg.horizontalCenter
                    anchors.top: iconBg.top
                    anchors.topMargin: -10 * main.vScaleFactor
                    sourceName: "Diagnostics/TV_menus_cursor";
                    z: channelListGrid.z + 1
                }

                Rectangle{
                    id: iconBg
                    color: "white"
                    width: parent.width
                    height: width

                    ScalableImage {
                        id: iconImage
                        anchors.fill: parent
                        fillMode: Image.PreserveAspectFit
                        sourceName: icon ? ("Diagnostics/channelIcons/" + icon).replace(/\.[^/.]+$/, "") : "Diagnostics/channelIcons/tv_default"

                        NumberLabel {
                            anchors.centerIn: parent

                            text: number
                            color: main.colorFontDark
                            font.pixelSize: 40 * main.vScaleFactor
                            font.bold: true
                            horizontalAlignment: Text.AlignHCenter
                        }

                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                if (channelListGrid.currentIndex === index){
                                    showIconSelector()
                                } else {
                                    root.textActive = true
                                    channelListGrid.currentIndex = index
                                }
                                if (root.multipleSelectEnabled)
                                    tv.channelList.toggleSelection(index)
                                    //tv.channelList.toggleSelection(index)

                                else {
                                   // tv.channelList.changeSelection(index)
                                    tv.channelList.changeSelection(index)
                                }
                            }
                            Rectangle {
                                anchors.fill: parent
                                color: main.colorGenericStimulusArea
                                opacity: 0.3
                                visible: main.showStimulusArea
                            }
                        }
                    }
                }

                TextInput {
                    id: channelLabelText
                    anchors { horizontalCenter: iconBg.horizontalCenter; top: iconBg.bottom; topMargin: 0 }
                    verticalAlignment: Text.AlignVCenter
                    text: name
                    font.pixelSize: 12 * main.vScaleFactor
                    width: parent.width
                    height: 20 * main.vScaleFactor
                    horizontalAlignment: Text.AlignHCenter
                    readOnly: !root.isSelectable
                    color: "white"

                    onActiveFocusChanged: {
                        if (root.isSelectable && activeFocus ) {
                            if(channelListGrid.currentIndex !== index ) {
                                root.textActive = true
                                channelListGrid.currentIndex = index
                            }
                        } else if(channelListGrid.currentIndex !== index ){
                            //tv.channelList.channels.setProperty(index, "name", text)
                            tv.channelList.channels.setProperty(index, "name", text)

                            //tv.channelList.saveChannelFile()
                            tv.channelList.doSave()
                        }
                    }
                    Keys.onPressed: {
                       switch(event.key){
                        case Qt.Key_Return:
                            tv.channelList.channels.setProperty(index, "name", text)
                            tv.channelList.doSave()
                           break;
                       }
                    }

                    Rectangle {
                        color: root.isSelectable ? Qt.rgba("30", "30", "30", 0.3) : "transparent"
                        border.color: root.isSelectable ? main.colorDiagnosticEdges : "transparent"
                        anchors.fill: parent
                        z: parent.z-1
                    }
                }
            }
        }
    }

    TrueKeyboard {
        id: keyboard
        width: root.width
        anchors.top: root.bottom
        z: 999999
        visible: root.isSelectable && root.textActive
        useMain: false
    }

    /*IconSelectorPopup {
        id: iconSelector
        blockKeys: true
        visible: false

        onIconSelected: {
            channels.setProperty(channelListGrid.imageClickIndex, "icon", iconSource)
            root.doSave() //save after we assign an icon
            hide()
        }

        onExitSelected: {
            hide()
        }
    }*/
}
