import QtQuick 2.12
import Process 0.1
import FileIO 1.0
import "../HomeScreen"
import "../string_format.js" as Format
import ".."
import Components 1.0

Item {
    id: root
    state: "NoUsbState"

    anchors.fill: parent

    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded
    property int numberOfImages
    property string cursorState: "controls" // controls is the only state currently

    function loadImageList() {
        var images = storageMgr.listFiles(slidesFileManager.slidesSource, ["*.png", "*.jpg"])
        root.numberOfImages = images.length
        slidesModel.clear()
        for (var i = 0; i < images.length; i++) {
            slidesModel.append({ file: slidesFileManager.slidesSource + "/" + images[i] })
        }
    }

    function doSave() {
        slidesFileManager.commitSlides()
        dialogText.text = qsTr("Slides have been updated") + application.emptyString
        actionSuccessfulDialog.show()
        actionSuccessfulDialog.startTimeout(1)
    }

    function doClearSlides() {
        slidesFileManager.deleteCurrentSlides()
        dialogText.text = qsTr("Slides have been cleared") + application.emptyString
        actionSuccessfulDialog.show()
        actionSuccessfulDialog.startTimeout(1)
    }

    onStateChanged: {
        if (state === "FoundUsbState") {
            loadImageList()
        }
    }

    onIsUsbDeviceLoadedChanged: {
        if (isUsbDeviceLoaded)
            state = "FoundUsbState"
        else
            state = "NoUsbState"
    }

    TextLabel {
        id: title
        text: qsTr("FACILITY IMAGES")
        color: main.colorFontDark
        anchors.top: parent.top
        anchors.topMargin: 30 * main.vScaleFactor
        anchors.horizontalCenter: parent.horizontalCenter
        font.pixelSize: 24 * main.vScaleFactor
    }

    SlidesFileManager {
        id: slidesFileManager
        slidesDestination: "/data/facility/user"
        slidesSource: "/media/usb/facility"
    }

    ListModel {
        id: slidesModel
    }

    Text {
        id: previewText
        font { pixelSize: 30 * main.vScaleFactor }
        text: qsTr("Preview") + application.emptyString
        anchors.top: title.bottom
        anchors.topMargin: 10 * main.vScaleFactor
        anchors.left: parent.left
        anchors.leftMargin: 50 * main.hScaleFactor
    }

    Slideshow {
        id: previewSlideshow
        width: parent.width * 0.5
        height: parent.height * 0.5
        anchors.top: previewText.bottom
        anchors.topMargin: 5 * main.vScaleFactor
        anchors.left: parent.left
        anchors.leftMargin: 50 * main.hScaleFactor
        imageList: slidesModel
        useCustomModel: true
        showControls: true
        slideInterval: 3000
        clip: true
    }

    Text {
        id: slideshowCounter

        visible: previewSlideshow.imageIndex >= 0 && state === "FoundUsbState"
        anchors.horizontalCenter: previewSlideshow.horizontalCenter
        anchors.top: previewSlideshow.bottom
        font { pixelSize: 24 * main.vScaleFactor }
        text: qsTr("Image") + " " + (1 + previewSlideshow.imageIndex) + " / " + previewSlideshow.imageList.count + application.emptyString
    }


    Text {
        id: insertUsbText
        anchors.top: title.bottom
        anchors.topMargin: 20 * main.vScaleFactor
        anchors.horizontalCenter: parent.horizontalCenter
        font { pixelSize: 24 * main.vScaleFactor }
        text: qsTr("INSERT USB KEY TO ADD IMAGES...") + application.emptyString
    }

    Text {
        id: foundText
        anchors.top: previewSlideshow.top
        anchors.topMargin: 0
        anchors.left: previewSlideshow.right
        anchors.leftMargin: 50 * main.hScaleFactor
        font { pixelSize: 24 * main.vScaleFactor }
        text: qsTr("Images Found") + ": " + numberOfImages + application.emptyString
    }

    StandardButton {
        id: saveButton

        buttonText: qsTr("SAVE?") + application.emptyString
        width: 120 * main.hScaleFactor
        height: 78 * main.vScaleFactor
        anchors.right: parent.right
        anchors.rightMargin: 20 * main.hScaleFactor
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 20 * main.vScaleFactor

        icon.anchors.topMargin: 5 * main.vScaleFactor
        icon.anchors.bottomMargin: 30 * main.vScaleFactor
        icon.sourceName: "WF/WF_Button_Saved_Workouts"
        onClicked: {
            root.doSave()
        }
    }

    ModalDialog {
        id: actionSuccessfulDialog

        Text {
            id: dialogText
            anchors.fill: parent
            anchors.verticalCenter: parent.verticalCenter
            color: main.colorFontLight
            anchors.horizontalCenter: parent.horizontalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            wrapMode: Text.WordWrap
            font.bold: false
            font.pixelSize: 50 * main.vScaleFactor
            z: parent.z + 1
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    Item {
        id: missingFolderText
        anchors.top: foundText.bottom
        anchors.topMargin: 20 * main.vScaleFactor
        anchors.left: foundText.left
        visible: true

        Text {
            width: 250 * main.hScaleFactor
            text: qsTr("Create a folder on your USB thumb drive titled \"facility\" (case sensitive). Any JPG or PNG file in that folder can be uploaded here.") + application.emptyString
            font.pixelSize: 14 * main.vScaleFactor
            wrapMode: Text.WordWrap
        }
    }

    StandardButton {
        id: clearButton

        width: 100 * main.hScaleFactor
        height: 78 * main.vScaleFactor
        anchors.right: parent.right
        anchors.rightMargin: 20 * main.hScaleFactor
        anchors.bottom: saveButton.top
        anchors.bottomMargin: 20 * main.vScaleFactor

        buttonText: qsTr("Clear") + application.emptyString
        onClicked: root.doClearSlides()
    }

    FileIO {
        id: fileio
        onError: print("IO error: " + error)
    }

    states: [
        State {
            name: "NoUsbState"
            PropertyChanges { target: insertUsbText; visible: true }
            PropertyChanges { target: foundText; visible: false }
            PropertyChanges { target: previewText; visible: false }
            PropertyChanges { target: previewSlideshow; visible: false }
            PropertyChanges { target: saveButton; visible: false }
        },

        State {
            name: "FoundUsbState"
            PropertyChanges { target: insertUsbText; visible: false }
            PropertyChanges { target: foundText; visible: true }
            PropertyChanges { target: previewText; visible: true }
            PropertyChanges { target: previewSlideshow; visible: true }
            PropertyChanges { target: saveButton; visible: true }
        }
    ]

    transitions:  [
        Transition {
            from: "NoUsbState"
            to: "FoundUsbState"
        }
    ]
}
