import QtQuick 2.12
import Components 1.0

Item {
    id: root

    property string cursorState: "log" // log or clear

    Component.onCompleted: {
        loadUsageIntoModel()
    }

    function loadUsageIntoModel() {
        var usage = workoutStats.getUsageAll()

        usageData.clear()

        if(!usage) {
            usageData.append({name:qsTr("NO USAGE DATA"), count: 0})
            return
        }

        for (var i = 0; i < usage.length; i++) {
            usageData.append({name: usage.item(i).name, count: usage.item(i).count})
        }
    }

    function clearStats() {
        workoutStats.clearAll()
        loadUsageIntoModel()
    }

    ListModel {
        id: usageData
    }

    Rectangle {
        id: dataScreen
        color: main.colorDialogBackground
        radius: main.defaultRadius
        width: 0.7 * root.width
        height: 0.7 * root.height
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        anchors.verticalCenterOffset: -30 * main.vScaleFactor

        ListView {
            id: dataTable

            property int itemsToShow: 13
            property bool moreItemsToScroll: currentIndex < dataTable.count - dataTable.itemsToShow

            function scrollPageUp() {
                dataTable.currentIndex = Math.max(0, dataTable.currentIndex - dataTable.itemsToShow)
            }

            function scrollPageDown() {
                dataTable.currentIndex = Math.min(dataTable.count - dataTable.itemsToShow,
                                                  dataTable.currentIndex + dataTable.itemsToShow)
            }

            clip: true
            anchors.fill: parent
            anchors.topMargin: 20 * main.vScaleFactor
            highlightRangeMode: ListView.ApplyRange
            preferredHighlightBegin: 0
            preferredHighlightEnd: 40

            model: usageData
            delegate: Component {
                Item {
                    width: parent.width
                    height: 40 * main.vScaleFactor

                    Text {
                        id: dataName
                        width: 0.8 * parent.width
                        height: parent.height
                        anchors.left: parent.left
                        anchors.leftMargin: 0.05 * parent.width
                        text: model.name
                        font.pixelSize: 28 * main.vScaleFactor
                        color: main.colorFontLight
                    }

                    NumberLabel {
                        id: dataCount
                        visible: model.count
                        width: 0.15 * parent.width
                        height: parent.height
                        anchors.right: parent.right
                        text: model.count
                        font.pixelSize: 28 * main.vScaleFactor
                    }
                }
            }
        }

        Rectangle {
            id: sliderBar

            property double baseHeight: dataTable.height
            property double contentHeight: dataTable.contentHeight

            anchors.right: parent.right
            anchors.rightMargin: 2 * main.hScaleFactor
            width: 15 * main.hScaleFactor
            height: baseHeight * baseHeight/contentHeight
            visible: baseHeight < contentHeight
            color: "darkgrey"
            radius: main.defaultRadius
            y: (baseHeight  - height) * dataTable.contentY / (contentHeight - baseHeight)
        }
    }

    StandardButton {
        id: clearButton

        width: 100 * main.hScaleFactor
        height: 78 * main.vScaleFactor
        anchors.right: parent.right
        anchors.rightMargin: 20 * main.hScaleFactor
        anchors.bottom: root.bottom
        anchors.bottomMargin: 20 * main.vScaleFactor

        buttonText: qsTr("Clear") + application.emptyString
        onClicked: root.clearStats()
    }
}
