import QtQuick 2.12
import ".."
import "DiagnosticsMenu.js" as DiagnosticsMenu
import Components 1.0

Item {
    id: root

    /**
     * Content loaded to the right of the menu
     */
    property alias content: page_content.sourceComponent
    property alias contentItem: page_content.item
    property alias contentSource: page_content.source

    property bool isSetupWizard: false

    property int topMargin: 0
    property variant menu: emptyMenu

    // Calling method needs to set menudef to the appropriate menu
    property variant menudef: DiagnosticsMenu.diagnosticsMenuUtilities()
    property bool showMenu: false

    function setTopMenu() {
        menu = emptyMenu
        showMenu = false

        if (root.menudef !== null && root.menudef.children !== undefined) {
            var menuitem
            menu.clear()
            for (var i=0; i < root.menudef.children.count ; ++i ) {
                menuitem = root.menudef.children.get(i)
                var newitem = {
                    "label" : application.translateText("Diagnostics", menuitem.label),
                    "qmlSource": menuitem.qmlSource
                }
                menu.append(newitem)
            }

            if(menu.count > 0)
                showMenu = true
        }
    }

    property alias menuIndex: sideMenuListView.currentIndex

    property int currentSelectedIndex: -1

    function menureset(){
        setTopMenu()
        menuIndex = 0
        root.currentSelectedIndex = 0
        contentSource = "DiagnosticsSummary.qml"
    }

    function updateStrings () {
        setTopMenu()
    }

    // re-translate the menu strings when the language changes
    Connections {
        target: application
        onLanguageChanged: { updateStrings() }
    }

    Component.onCompleted: {  // set the top level menu at startup
        menureset()
    }

    onMenuIndexChanged: {
        if(menu.count > 0)
            root.contentSource = menu.get(root.menuIndex).qmlSource
    }

    onMenudefChanged: {
        setTopMenu()
    }

    ListModel {
        id: emptyMenu
        ListElement {label: "Summary"; qmlSource: "DiagnosticsSummary.qml"}
    }

    Component{
        id: menuItemDelegate

        StandardButton {
            selected: root.menuIndex == index
            enabled: model.label !== "Synchronize Settings" || netConnections.wiredEnabled || netConnections.wirelessEnabled
            buttonText: model.label !== undefined ? qsTranslate("Diagnostics", model.label) + application.emptyString : ""
            label.font.pixelSize: 16 * main.vScaleFactor
            width: 0.8 * parent.width
            height: 70 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            onClicked: {
                if(!selected) {
                    buzzer.buttonPress()
                    root.menuIndex = index
                }
            }
        }
    }

    // side Menu
    Item{
        id:sideMenuSection
        anchors.left: root.left
        anchors.top: root.top
        anchors.bottom: root.bottom
        width: root.showMenu ? 0.2 * main.screenWidth : 0
        visible: root.showMenu
        z: parent.z + 1

        Rectangle {
            id: menuBackground
            color: "#808080" // "grey"
            anchors.fill: parent
        }

        ScrollBox {
            id: sideMenuListView
            anchors{
                top: parent.top
                topMargin: root.topMargin
                bottom: parent.bottom
                bottomMargin: 20 * main.vScaleFactor
                left: parent.left
                right: parent.right
            }

            color: "transparent"
            spacing: 20 * main.vScaleFactor
            model: menu
            delegate: menuItemDelegate
        }
    }

    // Main content
    Loader {
        id: page_content

        anchors {
            left: root.showMenu ? sideMenuSection.right : parent.left
            right: parent.right
            top: parent.top
            bottom: parent.bottom
            margins: 0.02 * parent.width
            topMargin: 0.03 * parent.height
        }

        clip: true
        onLoaded: {
            if(item.hasOwnProperty("isSetupWizard")) {
                item.isSetupWizard = root.isSetupWizard
            }
        }
    }
}
