import QtQuick 2.12
import StatisticManager 1.0
import Components 1.0

Item{
    id: root
    anchors.fill: parent
    anchors.margins: 20 * main.scaleFactor

    property string beltMotorUsageMilesLabel: qsTr("Belt Motor Usage in Miles") + application.emptyString
    property string beltMotorUsageSecondLabel: qsTr("Belt Motor Usage Time") + application.emptyString
    property string inclineMotorUsageSecondLabel: qsTr("Incline Motor Usage Time") + application.emptyString
    property string staticBrakeEnagagementCountLabel: qsTr("Static Brake Engagement Count") + application.emptyString
    property string staticBrakeUsageRevolutionsLabel: qsTr("Static Brake Usage in Revolutions") + application.emptyString

    property string pedalUsageMileLabel: qsTr("Distance") + application.emptyString
    property string pedalRevolutionsLabel: qsTr("Pedal Revolutions") + application.emptyString
    property string strideMotorUsageSecondLabel: qsTr("Stride Motor Usage Time") + application.emptyString

    property string stepCountLabel: qsTr("Steps") + application.emptyString
    property string stepUsageSecondLabel: qsTr("Stepper Motor Usage Time") + application.emptyString

    property string runHoursValue: Number(statisticManager.runHours/3600).toFixed(1) + " " + qsTr("Hours") + application.emptyString
    property string machineDistanceValue: options.metricUnits ? Globals.mileToKm(statisticManager.totalDistance/100).toFixed(2) + " " + qsTr("KM") : (statisticManager.totalDistance/100).toFixed(2) + " " + qsTr("miles") + application.emptyString
    property string distanceValue: options.metricUnits ? Globals.mileToKm(statisticManager.mileage/100).toFixed(2) + " " + qsTr("KM") : (statisticManager.mileage/100).toFixed(2) + " " + qsTr("miles") + application.emptyString

    property bool isTreadmill: machineinfo.productTypeString === "Treadmill"
    property bool isStepmill: machineinfo.productTypeString === "Stepmill"
    property bool isPedalSystem: {
        if(machineinfo.productTypeString.search("Elliptical") != -1 ||
                machineinfo.productTypeString.search("Bike") != -1) {
            return true
        } else {
            return false
        }
    }

    Component.onCompleted: {
        statisticManager.getNucleusStatistics();
    }

    function convertToTime(secs) {
        var hrs = (secs/3600).toFixed(0)
        var mins = (secs - hrs*3600)/60
        var str = hrs + qsTr(" Hrs. ") + mins.toFixed(0) + qsTr(" Mins. ")
        return str
    }

    Grid {
        id: infoBlock
        property int fontSize: 24 * main.vScaleFactor
        property int colWidth1: 0.68*width
        property int colWidth2: 0.32*width - columnSpacing
        visible: isTreadmill

        rowSpacing: 20 * main.vScaleFactor
        columnSpacing: 40 * main.hScaleFactor
        columns: 2
        rows: 5
        verticalItemAlignment: Grid.AlignVCenter

        width: parent.width
        height: parent.height
        anchors.top: parent.top
        anchors.topMargin: 50 * main.vScaleFactor

        TextLabel{
            text: beltMotorUsageMilesLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
            wrapMode: Text.WordWrap
        }
        NumberLabel{
            text: statisticManager.beltMotorUsageMile.toFixed(2)
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
        }

        TextLabel{
            text: beltMotorUsageSecondLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
            wrapMode: Text.WordWrap
        }
        NumberLabel{
            text: convertToTime(statisticManager.beltMotorUsageSecond)
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
        }

        TextLabel{
            text: inclineMotorUsageSecondLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
            visible: diagnosticsMain.accessLevel >= 1
            wrapMode: Text.WordWrap
        }
        NumberLabel{
            text: convertToTime(statisticManager.inclineMotorUsageSecond)
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
            visible: diagnosticsMain.accessLevel >= 1
        }

        TextLabel{
            text: staticBrakeEnagagementCountLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
            visible: diagnosticsMain.accessLevel >= 1
            wrapMode: Text.WordWrap
        }
        NumberLabel{
            text: statisticManager.staticBrakeEngagementCount
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
            visible: diagnosticsMain.accessLevel >= 1
        }

        TextLabel{
            text: staticBrakeUsageRevolutionsLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
            visible: diagnosticsMain.accessLevel >= 1
            wrapMode: Text.WordWrap
        }
        NumberLabel{
            text: statisticManager.staticBrakeUsageRevolutions
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
            visible: diagnosticsMain.accessLevel >= 1
        }
    }

    Grid {
        id: infoBlockPedalSystem
        property int fontSize: 24 * main.vScaleFactor
        property int colWidth1: 0.70*width
        property int colWidth2: 0.30*width - columnSpacing
        visible: isPedalSystem === true

        rowSpacing: 20 * main.vScaleFactor
        columnSpacing: 40 * main.hScaleFactor
        columns: 2
        rows: 3
        verticalItemAlignment: Grid.AlignVCenter

        width: parent.width
        height: parent.height
        anchors.top: parent.top
        anchors.topMargin: 50 * main.vScaleFactor

        TextLabel{
            text: pedalUsageMileLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
        }
        NumberLabel{
            text: options.metricUnits ? Globals.mileToKm(statisticManager.pedalUsageMile).toFixed(2) + " " + qsTr("KM") : (statisticManager.pedalUsageMile).toFixed(2) + " " + qsTr("Miles") + application.emptyString
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
        }

        TextLabel{
            text: pedalRevolutionsLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
        }
        NumberLabel{
            text: statisticManager.pedalRevolutions
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
        }

        TextLabel{
            text: strideMotorUsageSecondLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
            visible: diagnosticsMain.accessLevel >= 1 && main.isCSA
        }
        NumberLabel{
            text: convertToTime(statisticManager.strideMotorUsageSecond)
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
            visible: diagnosticsMain.accessLevel >= 1 && main.isCSA
        }
    }

    Grid {
        id: infoBlockStepmill
        property int fontSize: 24 * main.vScaleFactor
        property int colWidth1: 0.70*width
        property int colWidth2: 0.30*width - columnSpacing
        visible: isStepmill === true

        rowSpacing: 20 * main.vScaleFactor
        columnSpacing: 40 * main.hScaleFactor
        columns: 2
        rows: 3
        verticalItemAlignment: Grid.AlignVCenter

        width: parent.width
        height: parent.height
        anchors.top: parent.top
        anchors.topMargin: 50 * main.vScaleFactor

        TextLabel{
            text: stepCountLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
        }
        NumberLabel{
            text: statisticManager.stepUsageCount
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
        }

        TextLabel{
            text: stepUsageSecondLabel
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth1
        }
        NumberLabel{
            text:  convertToTime(statisticManager.stepSecondCount)
            font.pixelSize: parent.fontSize
            color: main.colorFontDark
            width: parent.colWidth2
        }
    }
}
