import QtQuick 2.12
import ".."
import Process 0.1
import Components 1.0
import FileIO 1.0

Item {
    id: root
    anchors.fill: parent

    property int fontSize: 16 * main.vScaleFactor
    property int textWidth: 400 * main.hScaleFactor
    property bool isSetupWizard: false
    property bool showPkgList: false
    property bool showLowerboardInfoList: false
    property string systemChecksum
    property string biosVersion

    onVisibleChanged: {
        if(!visible)
            import_exportMessage.hide()
    }

    Item {
        id: headingSection

        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: 100 * main.vScaleFactor

        TextLabel{
            id: heading
            text: qsTranslate("Diagnostics", "SUMMARY") + application.emptyString
            font.pixelSize: 36 * main.vScaleFactor
            anchors.fill: parent
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            color: main.colorFontDark
        }

        NumberLabel{
            id:serialNumber
            font.pixelSize: 12 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 3 * main.vScaleFactor
            color: main.colorFontDark

            text:qsTr("Serial: ") + main.serialNumber + "   " + qsTr("Version: ") + application.appVersion.split(".g")[0] + " / " + main.productVersion +  application.emptyString
        }
    }

    Grid {
        id: summaryArea
        property int fontSize: 14 * main.vScaleFactor
        property int colWidth: (parent.width - columnSpacing)/2

        rowSpacing: 20 * main.vScaleFactor
        columnSpacing : 40 * main.hScaleFactor
        columns: 2
        rows: 10
        verticalItemAlignment: Grid.AlignVCenter

        anchors.horizontalCenter: root.horizontalCenter
        anchors.horizontalCenterOffset: -width/6
        anchors.top: headingSection.bottom
        anchors.topMargin: 30 * main.vScaleFactor
        spacing: 40 * main.vScaleFactor

        TextLabel{
            text: qsTr("Product Model") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        TextLabel{
            text: (machineinfo.modelString + " " + machineinfo.productTypeString) + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark
        }

        TextLabel{
            text: qsTr("Serial Number") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        NumberLabel{
            text: main.serialNumber
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark
        }

        TextLabel  {
            text: qsTr("Machine Serial Number") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        NumberLabel{
            text: (main.lowerSystem === "NUCLEUS" ? (machineinfo.serialNumber) : qsTr("Not Available") ) + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark
        }

        TextLabel{
            text: qsTr("Tuner Type") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        TextLabel{
            text: tv.tunerType ? tv.tunerType : qsTr("Not Installed") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark
        }

        TextLabel{
            text: qsTr("Software Version") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        NumberLabel{
            text: application.appVersion
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark
        }

        TextLabel{
            visible: machineinfo.nucleusFirmwareVersion !== ""
            text: qsTr("Nucleus Firmware Version") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        NumberLabel{
            visible: machineinfo.nucleusFirmwareVersion !== ""
            text: machineinfo.nucleusFirmwareVersion
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark
        }

        TextLabel{
            text: qsTr("Product Version") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        NumberLabel{
            text: main.productVersion
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark
        }

        TextLabel{
            text: qsTr("BIOS Version") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        NumberLabel{
            text: root.biosVersion
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark
        }

        TextLabel{
            visible: false // hide this for now to save screen space
            text: qsTr("Bootloader Version") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }

        TextLabel{
            text: qsTr("System Software Sum") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        NumberLabel{
            text: root.systemChecksum
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    root.showPkgList = true
                    buzzer.buttonPress()
                }

                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }

        TextLabel{
            id: labelNucleusLowerboardInfo
            visible: main.lowerSystem === "NUCLEUS" ? true : false
            text: qsTr("Lowerboard Info") + application.emptyString
            font.pixelSize: fontSize
            width: textWidth
            horizontalAlignment: Text.AlignRight
            color: main.colorFontDark
        }
        Text{
            visible: labelNucleusLowerboardInfo.visible
            text: qsTr("Click to show") + application.emptyString
            font.pixelSize: fontSize
            width: paintedWidth
            horizontalAlignment: Text.AlignLeft
            color: main.colorFontDark

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    root.showLowerboardInfoList = true
                    buzzer.buttonPress()
                }

                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }

    }

    Column {
        visible: !root.isSetupWizard && diagnosticsMain.accessLevel >= 1
        anchors.right: root.right
        anchors.rightMargin: 20 * main.hScaleFactor
        anchors.bottom: root.bottom
        anchors.bottomMargin: 20 * main.hScaleFactor
        spacing: 15 * main.vScaleFactor

        StandardButton {
            id: importButton
            height: 60 * main.vScaleFactor
            enabled: storageMgr.usbDeviceLoaded
            buttonText: qsTr("IMPORT ALL SETTINGS") + application.emptyString

            onClicked: {
                importProcess.start()
            }
        }

        StandardButton {
            id: exportButton
            height: 60 * main.vScaleFactor
            enabled: storageMgr.usbDeviceLoaded
            buttonText: qsTr("EXPORT ALL SETTINGS") + application.emptyString

            onClicked: {
                exportProcess.start()
            }
        }

        StandardButton {
            id: factoryResetButton
            height: 60 * main.vScaleFactor
            buttonText: qsTr("RESET FACTORY DEFAULTS") + application.emptyString

            onClicked: {
                resetConfirmationPrompt.show()
            }
        }
    }

    MessagePopup {
        id: rebootPrompt
        message:qsTr("Power Cycle Required to Complete Update")
        visible: main.rebootRequired && root.visible

        StandardButton {
            property bool showButton: true
            visible: showButton
            z: parent.z + 1
            height: 50 * main.vScaleFactor
            anchors {
                horizontalCenter: parent.horizontalCenter
                bottom: parent.bottom
                bottomMargin: 25 * main.vScaleFactor
            }

            buttonText: qsTr("Reboot Now")
            onClicked: {
                main.startReboot()
                showButton = false
                rebootPrompt.message = qsTr("Rebooting now!")
            }
        }
    }

    ConfirmationPrompt {
        id: resetConfirmationPrompt
        visible: false
        prompt: qsTr("Are you sure?\nALL setup will be cleared.") + application.emptyString

        function deleteAttractVideos() {
            var destination = "/data/video/attract/user"
            var filesToDelete = storageMgr.listFiles(destination, ["*.mpg", "*.mpeg", "*.avi", "*.mp4", "*.jpg", "*.jpeg", "*.png", "*.bmp"])
            for (var i = 0; i < filesToDelete.length; i++) {
                var fullpath = destination + "/" + filesToDelete[i]
                var res = fileio.remove(fullpath)
                if (!res) {
                    console.log("!! file remove failed", source)
                }
            }
        }

        onYesClicked: {
            statisticManager.resetAll();
            errorManager.clearAllError()
            options.resetToDefaults()
            tv.channelList.clearChannels()
            tv.channelList.doSave()
            workoutStats.clearAll()
            miscStats.clearAll()
            slidesFileManager.deleteCurrentSlides()
            facilitySlidesManager.deleteCurrentSlides()
            deleteAttractVideos()
            resetEthernet.start()
            print("clearing network interfaces")
            clear_network_interfaces.start()
            resetSoftwareSources.start()
            resetSaltIDs.start()
            main.rebootRequired = true
            syncDisk.start()
            hide()
            //resetconnectionlist
        }

        onNoClicked: {
            hide()
        }
    }

    Process {
        id: resetEthernet

        onFinished: { dataExport.getNetworkAddresses() }

        program: "scripts/configure_network.sh"
        arguments: ["enp2s0"]
    }

    Process {
        id: clear_network_interfaces

        onFinished: { dataExport.getNetworkAddresses() }

        program: "scripts/clear_network_interfaces.sh"
    }

    // Disable any *daily.list or *testing.list in /etc/apt/sources.list.d
    Process {
        id: resetSoftwareSources
        program: "bash"
        arguments: "scripts/resetSoftwareSources.sh"
    }

    // Reset salt keys, minion_id
    Process {
        id: resetSaltIDs
        program: "bash"
        arguments: "scripts/salt/resetSaltIDs.sh"

    }

    Process {
        id: importProcess
        program: "scripts/importSettings.sh"
        arguments: [machineinfo.treadmillInclineMaxPercentGrade, machineinfo.treadmillSpeedMaxMph]
        onStarted: {
            import_exportMessage.isImport = true
            import_exportMessage.show()
        }

        onFinished: {
            import_exportMessage.hide()
            main.rebootRequired = true
        }

        onReadyReadStandardOutput: {
            console.log(readAllStandardOutput())
        }

        onReadyReadStandardError: {
            console.log(readAllStandardError())
        }
    }

    Process {
        id: exportProcess
        program: "scripts/exportSettings.sh"
        arguments: [args]

        property string args: {
            if(netConnections.wiredEnabled) {
                if(netConnections.wirelessEnabled)
                    return "both"
                else
                    return "wired"
            } else if(netConnections.wirelessEnabled)
                return "wifi"
            else
                return "none"
        }

        onStarted: {
            import_exportMessage.isImport = false
            import_exportMessage.show()
        }

        onFinished: {
            import_exportMessage.hide()
        }

        onReadyReadStandardOutput: {
            console.log(readAllStandardOutput())
        }

        onReadyReadStandardError: {
            console.log(readAllStandardError())
        }
    }

    ModalDialog {
        id: import_exportMessage
        visible: false

        property bool isImport: true

        Item {
            anchors.fill: parent
            z: parent.z + 1
            Text {
                anchors.fill: parent
                text: (import_exportMessage.isImport ? qsTr("Importing Settings. Please Wait.") : qsTr("Exporting Settings. Please Wait.")) + application.emptyString
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                font.pixelSize: 40 * main.vScaleFactor
                color: main.colorFontLight
            }
        }
    }

    SlidesFileManager {
        id: slidesFileManager
    }

    SlidesFileManager {
        id: facilitySlidesManager
        slidesDestination: "/data/facility/user"
    }

    Loader {
        id: packagelistView
        x: 200
        y: -20
        source: root.showPkgList ? "PackageListView.qml" : ""

        Connections {
            target: packagelistView.item
            ignoreUnknownSignals: true
            onClose: {
                print("close pkg view")
                root.showPkgList = false
            }
        }
    }

    Loader {
        id: lowerboardInfoView
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        source: root.showLowerboardInfoList ? "LowerboardInfoView.qml" : ""

        Connections {
            target: lowerboardInfoView.item
            ignoreUnknownSignals: true
            onClose: {
                root.showLowerboardInfoList = false
            }
        }
    }

    Process {
        id: getSystemChecksumProcess
        Component.onCompleted: start()
        program: "bash"
        arguments: [ "scripts/list_installed_packages.sh", "-s" ]
        onReadyReadStandardOutput: {
            root.systemChecksum = readAllStandardOutput()
            root.systemChecksum = root.systemChecksum.trim()
        }
    }

    Process {
        id: getBiosVerProcess

        Component.onCompleted: start()
        program: "sudo"
        arguments: ["cat", "/sys/class/dmi/id/bios_version"]
        onReadyReadStandardOutput: {
            var version = String(readAllStandardOutput())
            root.biosVersion = version.trim()
            print("Bios Version: " + root.biosVersion)
        }
    }

    FileIO {
        id: fileio
        onError: print("IO error: " + error)
    }
}
