import QtQuick 2.12
import ".."
import Components 1.0


Item {
    id: root

    anchors.fill: parent
    state: ""

    property bool isSetupWizard: false
    property bool isActive: (main.state === "DiagnosticState" && diagnosticsMain.currentSelectedIndex >= 0) || isSetupWizard

    property string calibratingMessageString: qsTr("Waiting...")

    signal close()
    signal inclineCalibrationComplete()

    TextLabel {
        id: title
        text:  qsTr("INCLINE CALIBRATION")
        color: main.colorFontDark
        anchors.top: parent.top
        anchors.topMargin: 30 * main.vScaleFactor
        anchors.horizontalCenter: parent.horizontalCenter
        font.pixelSize: 24 * main.vScaleFactor
    }

    Text {
        id: warningLabel
        text: qsTr("WARNING: Tread belt will move at max speed!")
        color: "red"
        font.pixelSize: 14 * main.vScaleFactor
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: title.bottom
        anchors.topMargin: 10 * main.vScaleFactor
    }

    Grid {
        id: infoBlock

        property int fontSize: 14 * main.vScaleFactor
        property int colWidth: (parent.width - columnSpacing)/2

        rowSpacing: 20 * main.vScaleFactor
        columnSpacing : 40 * main.hScaleFactor
        columns: 2
        rows: 5
        verticalItemAlignment: Grid.AlignVCenter

        width: parent.width
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: warningLabel.bottom
        anchors.topMargin: 50 * main.vScaleFactor

        TextLabel {
            text: qsTr("INCLINE RANGE") + application.emptyString
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignRight
        }

        TextLabel {
            text: (machineinfo.treadmillInclineMinPercentGrade / 10).toFixed(1) + "%" + qsTr(" to ") + (machineinfo.treadmillInclineMaxPercentGrade / 10).toFixed(1) + "%"
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignLeft
        }

        TextLabel {
            text: qsTr("INCLINE ADC RANGE") + application.emptyString
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignRight
        }

        TextLabel {
            text: machineinfo.treadmillCalibratedInclineMinAdc.toFixed(0) + qsTr(" to ") + machineinfo.treadmillCalibratedInclineMaxAdc.toFixed(0)
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignLeft
        }

        TextLabel {
            text: qsTr("ACTUAL INCLINE ADC") + application.emptyString
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignRight
        }

        TextLabel {
            text: machinecontrol.treadmillInclineADCActual
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignLeft
        }

        TextLabel {
            text: qsTr("ACTUAL SPEED") + application.emptyString
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignRight
        }

        TextLabel {
            text: (machinecontrol.treadmillSpeedActual / 10.0).toFixed(1)
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignLeft
        }

        TextLabel {
            text: qsTr("TEST RESULTS") + application.emptyString
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignRight
        }

        TextLabel {
            text: root.calibratingMessageString
            font.pixelSize: infoBlock.fontSize
            color: main.colorFontDark
            width: infoBlock.colWidth
            horizontalAlignment: Text.AlignLeft
        }
    }

    StandardButton{
        id: setZeroButton

        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: infoBlock.bottom
        anchors.topMargin: 50 * main.vScaleFactor
        width: 100 * main.hScaleFactor
        height: 50 * main.vScaleFactor
        enabled: machinecontrol.isCalibrating === false
        buttonText: qsTr("RUN") + application.emptyString

        beep:false
        onClicked: startCalibrationTimer.start()
    }

    Connections {
        target: machinecontrol
        onIsCalibratingChanged: {
            if(isCalibrating) {
                calibratingMessageString = qsTr("CALIBRATING...") + application.emptyString
            }
            else {
                calibratingMessageString = qsTr("CANCELLED...") + application.emptyString
            }
        }
        onCalibrationComplete: {
            if(result){
                calibratingMessageString = qsTr("CALIBRATION SUCCESSFUL") + application.emptyString
            }
            else {
               calibratingMessageString = qsTr("CALIBRATION FAILED") + application.emptyString
            }
        }
    }

    MessagePopup{
        id:goingToCalibrateMessage
        dismissable: false

    }

    Timer{
        id: startCalibrationTimer
        interval: 1000
        repeat: true
        running: false

        property int counter: 3

        function updateGoingToCalibrateMessage(){
            goingToCalibrateMessage.message = qsTr("STARTING CALIBRATION IN ") +"<br>" + counter + application.emptyString
        }

        onRunningChanged: if(running) {
                              buzzer.warningBeep()
                              goingToCalibrateMessage.show();
                              updateGoingToCalibrateMessage()
                          }
        onTriggered: {
            counter--;
            if(counter == 0){
                counter = 3
                startCalibrationTimer.stop()
                machinecontrol.calibrationStart()
                goingToCalibrateMessage.hide()
            }
            else
            {
                buzzer.warningBeep()
                updateGoingToCalibrateMessage();
            }
        }

    }

    //Component.onCompleted: machinecontrol.treadmillSpeedTarget = 5
    Component.onDestruction: machinecontrol.treadmillSpeedTarget = 0
}
