import QtQuick 2.12
import Components 1.0
//import NetworkRequest 1.0
import EGym 1.0
import "../Keypad"

Item {
    id: root

    anchors {
        fill: parent
        margins: 10 * main.hScaleFactor
    }

    property alias gymID: unameInput.text
    property alias gymPassword: upassInput.text

    Component.onCompleted: {
        eGym.validatePartner()
        gem.connectionAllowed = false
    }

    Component.onDestruction: {
        gem.connectionAllowed = true
    }

    onGymIDChanged: {
        options.egymID = gymID
        eGym.gymID = gymID
    }

    onGymPasswordChanged: {
        options.egymPassword = gymPassword
        eGym.gymPassword = gymPassword
    }

    Column {
        id: headingSection

        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: 150 * main.vScaleFactor
        spacing: 3 * main.vScaleFactor

        ScalableImage {
            id: eGymConnect_icon
            width: parent.width
            height: 70 * main.vScaleFactor
            fillMode: Image.PreserveAspectFit
            sourceName: "eGym/EGYMLogoIconDark"
        }

        TextLabel{
            id: heading
            text: qsTranslate("Diagnostics", "Registration") + application.emptyString
            anchors.horizontalCenter: parent.horizontalCenter
            font.pixelSize: 36 * main.vScaleFactor
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            color: main.colorFontDark
        }

        TextLabel{
            id: partnerName
            font.pixelSize: 18 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            color: main.colorFontDark

            text:qsTr("Partner Name: ") + eGym.partnerName + application.emptyString
        }

        TextLabel{
            id:serialNumber
            font.pixelSize: 12 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            color: main.colorFontDark

            text:qsTr("Serial: ") + main.serialNumber + "   " + qsTr("Type: ") + machineinfo.productTypeString + application.emptyString
        }
    }

    Column {
        spacing: 25 * main.hScaleFactor
        anchors {
            top: headingSection.bottom
            left: parent.left
            right: parent.right
            margins: 20 * main.hScaleFactor
            bottom: parent.bottom
        }

        TextLabel {
            id: unameLabel

            width: parent.width
            font.pixelSize: 16 * main.vScaleFactor
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignTop
            color: main.colorFontDark
            text: qsTr("Enter Gym ID:") + application.emptyString
        }

        TextInput {
            id: unameInput

            width: parent.width
            text: eGym.gymID

            Rectangle {
                color: "transparent"
                border.color: main.colorDiagnosticEdges
                anchors.fill: parent
                anchors.margins: -2*main.vScaleFactor
                z: parent.z-1
            }
        }

        TextLabel {
            id: upassLabel

            width: parent.width
            font.pixelSize: 16 * main.vScaleFactor
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignTop
            color: main.colorFontDark
            text: qsTr("Enter Gym Password:") + application.emptyString
        }

        TextInput {
            id: upassInput
            width: parent.width
            text: eGym.gymPassword
            echoMode: TextInput.Password

            Rectangle {
                color: "transparent"
                border.color: main.colorDiagnosticEdges
                anchors.fill: parent
                anchors.margins: -2*main.vScaleFactor
                z: parent.z-1
            }
        }

        StandardButton {
            id: validate
            anchors.horizontalCenter: parent.horizontalCenter
            buttonText: qsTr("Validate Registration")
            onClicked: {
                eGym.validatePartner()
            }
        }
    }

    ModalDialog {
        id: errorPopup

        visible: false
        dismissAnywhere: true
        dismissable: true
        countDown: 20
        z: 999

        onVisibleChanged: {
            if(visible) {
                startTimeout(20)
            }
        }

        onTimedOut: {
            errorPopup.hide()
        }

        CloseButton {
            id: closeButton

            z: parent.z + 1
            anchors {
                right: parent.right ; rightMargin: 80 * main.hScaleFactor
                top: parent.top ; topMargin: 40 * main.vScaleFactor
            }
            onClicked: errorPopup.hide()
        }

        TextLabel {
            z: parent.z + 1
            anchors.fill: parent
            id: errorLabel
            text: application.emptyString
            font.pixelSize: 25 * main.vScaleFactor
            width: parent.width
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
    }

    Connections {
        target: eGym
        onError: {
            errorLabel.text = qsTr("Error:") + errorStr + " (" + errorCode + ")" + application.emptyString
            errorPopup.show()
        }

        onPartnerInfoValidated: {
            errorLabel.text = qsTr("Registration Successful") + application.emptyString
            errorPopup.show()
        }
    }

    TrueKeyboard {
        id: keyboard
        width: root.width
        anchors.horizontalCenter: root.horizontalCenter
        z: 999999
        active: unameInput.activeFocus || upassInput.activeFocus
        useMain: false
    }
}
