import QtQuick 2.12
import Components 1.0
import Process 0.1
import ".."

ModalDialog {
    id: root

    property string preparingString: qsTr("Preparing Diagnostic Report...") + application.emptyString
    property string sendingString: qsTr("Sending Diagnostic Report...") + application.emptyString

    isTop: true
    isTransparent: false
    dismissable: false
    Component.onCompleted: if (!application.crashRecovery) hide()

    function start() {
        // Note: Check connection list directly to avoid having to wait on
        // NetworkConnections object to build its list
        var files = storageMgr.listFiles("/etc/network/interfaces.d", "*.cfg")
        if (!options.diagnosticsUploadEnabled || files.length <= 0) {
            // Feature disabled or no network configured - we are done
            removeCrash.start()
        } else
            autoLogUploader.start()
    }

    Text {
        width: 501 * main.hScaleFactor
        color: "#ffffff"
        text: autoLogUploader.running ? root.sendingString : root.preparingString
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        horizontalAlignment: Text.AlignHCenter
        wrapMode: Text.WordWrap
        font.bold: false
        font.pixelSize: 50 * main.vScaleFactor
        z: root.z + 1
    }

    LogUploader {
        id: autoLogUploader

        onProcessFinished: {
            root.hide()
            main.rebootRequired = true
        }

        onUploadFailed: {
            logger.logAlways("Automatic Log Upload: Upload Failed")
        }

        onTransferComplete: {
            logger.logAlways("Automatic Log Upload: Transfer Complete")
        }

        onTerminated: {
        }

        onConnectionError: {
            logger.logAlways("Automatic Log Upload: Connection Error")
        }
    }

    Process {
        id: rebootXXX
        program: "scripts/export-diagnostics/reboot.sh"

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput())
            console.log("[reboot.sh]", data)
        }
    }

    Process {
        id: removeCrash
        program: "rm"
        arguments: ["/tmp/truetouch.crash"]

        onFinished: {
            root.hide()
            main.rebootRequired = true
            console.log("Removed /tmp/truetouch.crash")
        }
    }

}
