import QtQuick 2.12
import Process 0.1
import Components 1.0
import "../"

Rectangle {
    id: root
    width: main.widthDialog
    height: alphaBar.height + infoLine.height + 5 * main.vScaleFactor

    y: 0.06 * main.screenHeight

    color: main.colorDialogBackground

    signal close()

    Component.onCompleted: listPackagesProcess.getList()

    CloseButton {
        id: closeButton

        anchors {
            left: parent.right
            leftMargin: 5 * main.hScaleFactor
            top: parent.top
        }
        onClicked: close()
    }

    ListView {
        id: listView

        function showInitial(letter) {
            var i = packageList.letterIndex(letter)
            listView.currentIndex = i
        }
        clip: true
        height: alphaBar.height
        anchors {
            top: parent.top
            left: parent.left
            right: parent.right
            leftMargin: 5 * main.hScaleFactor
            rightMargin: alphaBar.width + 10 * main.hScaleFactor
        }
        highlightRangeMode: ListView.ApplyRange
        preferredHighlightBegin: 0
//        preferredHighlightEnd: pkgNameLabel.paintedHeight
        highlightMoveVelocity: 10000000
        highlight: Component {
            Rectangle {
                color: "white"
                opacity: 0.5
            }
        }

        delegate: Item {
            width: parent.width
            height: pkgNameLabel.paintedHeight
            Text {
                id: pkgNameLabel
                text: pkgname ? pkgname : "" ; height: paintedHeight ; width: paintedWidth
                anchors.left: parent.left ; horizontalAlignment: Text.AlignLeft
                color: main.colorFontLight
            }
            Text {
                text: version ? version : "" ; height: paintedHeight ; width: paintedWidth
                anchors.right: parent.right ; horizontalAlignment: Text.AlignRight
                color: main.colorFontLight
            }
        }

        model: packageList
    }

    Rectangle {
        id: infoLine
        anchors { top: listView.bottom; topMargin: 5 * main.vScaleFactor }
        width: root.width
        height: title.paintedHeight
        color: "#cccccc"

        Text {
            id: title

            text: qsTr("Total Packages") + ":" + packageList.count
            width: paintedWidth
            height: paintedHeight
            color: main.colorFontDark
            font { pixelSize: 16 * main.vScaleFactor}

            anchors {
                right: parent.right
                top: parent.top
            }
        }
    }

    Column {
        id: alphaBar
        anchors {
            left: listView.right
            leftMargin: 10 * main.hScaleFactor
            top: parent.top
        }

        Repeater {
            model: 26
            delegate: Component {
                TextLabel {
                    text: String.fromCharCode(65 + index)
                    height: paintedHeight - 5 * main.vScaleFactor

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            listView.showInitial(parent.text.toLowerCase())
                            buzzer.buttonPress()
                        }

                        Rectangle {
                            anchors.fill: parent
                            color: main.colorGenericStimulusArea
                            opacity: 0.3
                            visible: main.showStimulusArea
                        }
                    }
                }
            }
        }
    }

    ListModel {
        id: packageList

        function letterIndex(letter) {
            for( var i=0 ; i < count ; i++ ){
                if(get(i).pkgname[0] >= letter)
                    return i
            }
            return count-1
        }
    }

    focus: true
    Keys.onPressed: {
        var i = packageList.letterIndex(event.text)
        print("index: ")
        listView.currentIndex = i
    }

    // The following process gathers a list of packages that may be updated
    Process {
        id: listPackagesProcess
        property string packages
        property string parseLine
        property string leftover

        function getList() {
            packageList.clear()
            parseLine = ""
            start()
        }

        program: "bash"
        arguments: [ "scripts/list_installed_packages.sh" ]
        onReadyReadStandardOutput: {
            parseLine += readAllStandardOutput()
            var isRemainder = false
            if(parseLine[parseLine.length-1] != '\n'){
                print("leftovers: " )
                isRemainder = true
            }
            var lines = parseLine.split(/\n/)
            var linecount = lines.length
            if(isRemainder)
                linecount--
            for(var i=0 ; i < linecount ; i++) {
                var line = lines[i]

                print(line.length + " line:"+ line)
                if( line.length <= 0 )
                    continue
                var fields = line.split(/\|/)
                var element = { "pkgname":fields[0], "version" : fields[1] }
                print("name:" + fields[0] + "  ver:" + fields[1])
                packageList.append(element)
            }
            parseLine = ""
            // save remainder partial line for next i/o event
            if(isRemainder) {
                print("remainder:" + lines[linecount])
                parseLine = lines[linecount]
            }
        }
        onStarted: console.log("listing packages")
        onFinished: {
            print("package list:")
            for(var i=0 ; i < packageList.count ; i++)
                print(packageList.get(i).package + "---" + packageList.get(i).version )
        }
    }
}
