import QtQuick 2.12
import "../Keypad"
import Components 1.0

ModalDialog {
    id: root

    property variant model: wifiScanner.accessPoints
    property bool hideWireless: false

    signal selected(variant accessPoint)
    signal canceled()

    state: "connect"

    opacity: 1

    onVisibleChanged: {
        if(visible && !hideWireless)
            wifiScanner.startScan()
    }

    Component.onCompleted: {
        if (hideWireless) {
            wifiScanner.accessPoints.insert(0, { essid:"Wired Ethernet", interface:"enp2s0", encrypted: false } )
        }
    }

    function connect() {
        buzzer.buttonPress()
        wifiList.selectedItem.password = passwordInput.text
        passwordInput.text = ""
        root.selected(root.model.get(wifiList.currentIndex))
        root.state = "connect"
    }

    Connections {
        target: wifiScanner
        onScanComplete: {
            // add the hardwired Ethernet option at top of list
            wifiScanner.accessPoints.insert(0, { essid:"Wired Ethernet", interface:"enp2s0", encrypted: false } )
        }
    }

    Item {
        id: mainControlsItem
        z: root.z + 1
        anchors.fill: parent

        ListView {
            id: wifiList

            property variant selectedItem

            onCurrentIndexChanged: selectedItem = model.get(currentIndex)

            model: root.model
            delegate: listDelegate

            clip: true
            width: 0.5 * parent.width
            height: 0.7 * parent.height
            anchors {
                left: parent.left
                leftMargin: 80 * main.hScaleFactor
                top: parent.top
                topMargin: 60 * main.vScaleFactor
            }

            highlight: Rectangle {
                color: main.colorHighlightBackground
                radius: 5 * main.scaleFactor
                opacity: 0.7
                width: 400
            }
            highlightMoveVelocity: 1500

            // gray background for wifi scan result list
            Rectangle {
                color: "darkgray" ;
                border { color: main.colorUIBorder ; width: main.widthBorder }
                anchors { fill: wifiList ; margins: -8 * main.scaleFactor }
                z: parent.z - 1
            }
        }

        Text {
            text: qsTr("Scanning for wireless networks...")
            visible: wifiScanner.scanning
            color: main.colorFontLight

            anchors {
                bottom: wifiList.top
                bottomMargin: 5 * main.vScaleFactor
                left: wifiList.left
            }
        }

        Component {
            id: listDelegate
            Item {
                width: wifiList.width
                height: accessPointDelegate.height

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        wifiList.currentIndex = index
                        buzzer.buttonPress()
                    }

                    Rectangle {
                        anchors.fill: parent
                        color: main.colorGenericStimulusArea
                        opacity: 0.3
                        visible: main.showStimulusArea
                    }
                }

                Row {
                    id: accessPointDelegate
                    spacing: 5 * main.hScaleFactor
                    ScalableImage {
                        sourceName: encrypted ? "HomeScreen/LOCK_ICON" : "Diagnostics/padlock-unlocked"
                    }
                    Text {
                        text: essid ? essid : ""
                        width: 170 * main.hScaleFactor
                        height: parent.height
                        anchors.verticalCenter: parent.verticalCenter
                        verticalAlignment: Text.AlignVCenter
                        font.pixelSize: 0.5 * height
                        color: main.colorFontLight
                    }
                }
            }
        }

        Column {

            property int buttonHeight: (height-2*spacing)/3

            spacing: 20 * main.vScaleFactor
            width: 0.2 * parent.width
            height: wifiList.height
            anchors {
                right: parent.right
                rightMargin: 50 * main.hScaleFactor
                top: wifiList.top
            }

            StandardButton {
                id: connectButton

                visible: wifiList.selectedItem ? true : false
                width: parent.width
                height: parent.buttonHeight

                buttonText: wifiList.selectedItem ? qsTr("Connect " + wifiList.selectedItem.essid) : ""
                onClicked: {
                    if(!wifiList.selectedItem.encrypted)
                        root.selected(root.model.get(wifiList.currentIndex))
                    else {
                        root.state = "password"
                    }
                }
            }

            StandardButton {
                id: scanButton

                visible: !hideWireless && !wifiScanner.scanning
                width: parent.width
                height: parent.buttonHeight

                buttonText: qsTr("Scan") + application.emptyString
                onClicked: {
                    wifiScanner.startScan()
                }
            }

            StandardButton {
                id: cancelButton

                width: parent.width
                height: parent.buttonHeight

                buttonText: qsTr("Cancel") + application.emptyString
                onClicked: {
                    root.canceled()
                }
            }
        }
    }

    Rectangle {
        id: password_popup

        anchors.fill: parent
        color: main.colorDialogBackground
        border.color: main.colorUIBorder
        border.width: main.widthBorder

        z: mainControlsItem.z+1
        visible: false

        Text {
            id: passwordLabel
            anchors {
                 horizontalCenter: parent.horizontalCenter
                 top: parent.top
                 topMargin: 30 * main.vScaleFactor
            }
            width: parent.width
            color: main.colorFontLight
            font.pixelSize: 18 * main.vScaleFactor

            text: qsTr("Enter Password")
            horizontalAlignment: Text.AlignHCenter
        }

        Rectangle {
            id: passwordEntry
            anchors {
                horizontalCenter: parent.horizontalCenter
                top: passwordLabel.bottom
                topMargin: 30 * main.vScaleFactor
            }
            width: 0.5 * parent.width
            height: 40 * main.vScaleFactor
            color: "white"
            border.color: main.colorDiagnosticEdges
            border.width: main.widthBorder

            TextInput {
                id: passwordInput

                anchors {
                    fill: parent
                    margins: 10 * main.scaleFactor
                }
                verticalAlignment: Text.AlignVCenter
                font.pixelSize: 12 * main.vScaleFactor
                text: ""
                inputMethodHints: Qt.ImhNoAutoUppercase
                echoMode: TextInput.Password

                onEditingFinished: connect()

                onActiveFocusChanged: {
                    if (activeFocus) {
                        text = ""
                    }
                }
            }
        }

        StandardButton {
            anchors {
                horizontalCenter: parent.horizontalCenter
                top: passwordEntry.bottom
                topMargin: 60 * main.vScaleFactor
            }
            buttonText: qsTr("Connect ")

            onClicked: connect()
        }

        CloseButton { // x
            anchors {
                top: parent.top
                topMargin: 20 * main.vScaleFactor
                right: parent.right
                rightMargin: 20 * main.hScaleFactor
            }

            onClicked: {
                buzzer.buttonPress()
                passwordInput.text = ""
                root.state = "connect"
            }
        }
    }

    TrueKeyboard {
        id: keyboard
        width: main.width * 0.75
        anchors.horizontalCenter: root.horizontalCenter
        z: 999999
        active: passwordInput.activeFocus
        hiddenY: root.height + ((main.height - height) / 2)
        useMain: false
    }

    WifiScan {
        id: wifiScanner
    }

    states: [
        State {
            name: "connect"
            PropertyChanges { target: password_popup; visible: false }
        },
        State {
            name: "password"
            PropertyChanges { target: password_popup; visible: true }
        }
    ]
}
