import QtQuick 2.12
import Components 1.0
import Process 0.1
import ".."

ConfirmationPrompt {
    id: root

    property bool active: root.visible || statusPopup.visible || noUpdatesFoundPopup.visible

    function showPrompt() {
        if (state != "Confirm" && state != "Finished") {
            return
        }

        state = "Confirm"
        show()
    }

    state: "Confirm"
    z: 999999
    onYesClicked: {
        if (ipod_window.visible) {
            ipod_window.close()
        }

        console.log("Update!")
        root.hide()
        statusPopup.show()
        state = "Disconnect" // Disconnect the network
    }

    Process {
        id: enableDisableNetwork
        program: "scripts/enable_disable_network.sh"
        arguments: ["0"] // 0=Off, 1=On
        onFinished: {
            if (state === "Disconnect") {
                state = "CheckUpdates"
            } else if (state === "Reconnect") {
                state = "Finished"
            } else if (state === "Error") {
                // Do nothing - Keep popup shown
            } else {
                console.log("enableDisableNetwork: Unknown state:", state)
            }
        }
    }

    Process { // The following process gathers a list of packages that may be updated
        id: checkUpdates

        property string packages

        program: "bash"
        arguments: [ "scripts/list_upgrades.sh" ]
        onReadyReadStandardOutput: packages += readAllStandardOutput()
        onStarted: {
            packages = ""
            console.log("checking updates")
        }
        onFinished: {
            console.log("packages upgradable:" + packages)

            if (packages) {
                state = "DoUpdates"
            } else {
                state = "Error" // No updates found
            }
        }
    }

    Process {
        id: updatePackages
        program: "bash"
        arguments: ["scripts/do_upgrades.sh"]
        onReadyReadStandardOutput: console.log(readAllStandardOutput())
        onReadyReadStandardError: console.log(readAllStandardError())
        onStarted: console.log("updating packages...")
        onFinished: {
            print("Diagnostics Update: package update complete.")
            state = "Reconnect"
            main.rebootRequired = true
        }
    }

    MessagePopup {
        id: statusPopup
        z: 999999
    }

    ModalDialog {
        id: noUpdatesFoundPopup
        visible: false
        z: 999999

        property string message: qsTr("No Updates Found") + application.emptyString

        Item {
            z: noUpdatesFoundPopup.z + 1
            anchors.fill: parent

            Text {
                id: popupText
                anchors {
                    fill: parent
                    margins: 30
                    horizontalCenter: parent.horizontalCenter
                    verticalCenter: parent.verticalCenter
                    verticalCenterOffset: -40
                }
                color: main.colorFontLight
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                font.bold: false
                font.pixelSize: 50
                text: noUpdatesFoundPopup.message
            }

            CloseButton { // x
                anchors {
                    right: parent.right
                    rightMargin: 5 * main.hScaleFactor
                    top: parent.top
                    topMargin: 5 * main.vScaleFactor
                }
                onClicked: {
                    noUpdatesFoundPopup.hide()

                    if (enableDisableNetwork.processState === Process.NotRunning) {
                        root.state = "Finished"
                    } else {
                        root.state = "Reconnect"
                    }
                }
            }
        }
    }

    Timer {
        id: hideReconnectPopupTimer
        interval: 10 * 1000
        repeat: false
        running: false
        onTriggered: root.state = "Finished"
    }

    states: [
        State {
            name: "Confirm"

            StateChangeScript {
                script: {
                    statusPopup.hide()
                }
            }
        },
        State {
            name: "Disconnect"

            PropertyChanges { target: statusPopup; message: QT_TR_NOOP("Disconnecting From Network") }
            StateChangeScript {
                script: {
                    if (enableDisableNetwork.processState != Process.NotRunning) {
                        enableDisableNetwork.kill()
                    }

                    enableDisableNetwork.arguments = ["0"]
                    enableDisableNetwork.start()
                }
            }
        },
        State {
            name: "CheckUpdates"

            PropertyChanges { target: statusPopup; message: QT_TR_NOOP("Checking For Updates") }
            StateChangeScript {
                script: {
                    checkUpdates.start()
                }
            }
        },
        State {
            name: "Error"

            StateChangeScript {
                script: {
                    statusPopup.hide()
                    enableDisableNetwork.arguments = ["1"] // Start reconnect early, don't show popup.
                    enableDisableNetwork.start()
                    noUpdatesFoundPopup.show()
                }
            }
        },
        State {
            name: "DoUpdates"

            PropertyChanges { target: statusPopup; message: QT_TR_NOOP("Updating Software...") }
            StateChangeScript {
                script: {
                    updatePackages.start()
                }
            }
        },
        State {
            name: "Reconnect"

            PropertyChanges { target: statusPopup; message: QT_TR_NOOP("Please Wait. Reconnecting to Network") }
            StateChangeScript {
                script: {
                    statusPopup.show()
                    hideReconnectPopupTimer.start()

                    if (enableDisableNetwork.processState === Process.NotRunning) {
                        enableDisableNetwork.arguments = ["1"]
                        enableDisableNetwork.start()
                    }
                }
            }
        },
        State {
            name: "Finished"

            StateChangeScript {
                script: {
                    noUpdatesFoundPopup.hide()
                    statusPopup.hide()
                }
            }
        }
    ]
}
