import QtQuick 2.12
import Components 1.0
import ".."

/**
 * @brief Rectangle that contains the Forgot Password components within the application.
 */
Rectangle {
    id: box
    anchors.fill: parent
    color: main.colorDialogBackground
    border.color: main.colorUIBorder
    border.width: main.widthBorder

    /**
     * @brief Signal emitted when box_button is clicked.
     */
    signal submitClicked()

    /**
     * @brief Determines if the text field box's content is acceptable for submission.
     * @return type:bool Returns true if acceptable for submission and false otherwise.
     */
    function isAcceptable() {
        return (box_cellfield.acceptableInput);
    }

    /**
     * @brief Gets the text within the text field's box.
     * @brief type:string Returns the text if acceptable and "NULL" otherwise.
     */
    function getInput() {
        return (box_cellfield.acceptableInput) ? box_cellfield.getText() : "NULL";
    }

    /**
     * @brief Clears the contents within the text box.
     */
    function clear() {
        box_cellfield.clear();
    }

    Item {
        anchors {
            fill: parent
            topMargin: 60 * main.vScaleFactor
            leftMargin: 20 * main.hScaleFactor
            rightMargin: 20 * main.hScaleFactor
            bottomMargin: 32 * main.vScaleFactor
        }

        Text {
            id: box_title
            anchors {
                left: parent.left
                top: parent.top
                right: parent.right
            }
            color: "white"
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignTop
            wrapMode: Text.Wrap
            font.pixelSize: 14 * main.vScaleFactor
            text: qsTr("ENTER ACCOUNT CELL PHONE NUMBER") + application.emptyString
        }

        Text {
            id: box_desc
            anchors {
                left: parent.left
                right: parent.right
            }
            y: 50 * main.vScaleFactor
            color: "white"
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignTop
            wrapMode: Text.Wrap
            font.pixelSize: 12 * main.vScaleFactor
            text: qsTr("A verification code will be sent to your phone.") + application.emptyString
        }

        StandardTextField {
            id: box_cellfield
            anchors.horizontalCenter: parent.horizontalCenter
            y: 120 * main.vScaleFactor
            width: 270 * main.hScaleFactor
            height: 42 * main.vScaleFactor
            placeholderText: qsTr("CELL PHONE NUMBER") + application.emptyString
            inputMethodHints: Qt.ImhDigitsOnly
            validator: RegExpValidator { regExp: /^\d{3,32}$/ }
        }

        StandardButton {
            id: box_button
            anchors {
                bottom: parent.bottom
                horizontalCenter: parent.horizontalCenter
            }
            buttonText: qsTr("NEXT") + application.emptyString
            textColor: main.colorFontGreen
            onClicked: {
                box.submitClicked()
                box.clear()
            }

            icon.sourceName: "Components/ArrowRightGreenIcon"
        }
    }
}
