import QtQuick 2.12
import Components 1.0
import ".."

/**
 * @brief Rectangle that contains the Sign-In components within the application.
 */
Rectangle {
    id: box
    anchors.fill: parent
    color: main.colorDialogBackground
    border.color: main.colorUIBorder
    border.width: main.widthBorder

    /**
     * @brief Signal emitted when box_button is clicked.
     */
    signal submitClicked()

    /**
     * @brief Determines if the text fields content is acceptable for submission.
     * @return type:bool Returns true if both text field's text is acceptable and false otherwise.
     */
    function isAcceptable() {
        return ((box_cellfield.acceptableInput) && (box_passfield.acceptableInput));
    }

    /**
     * @brief Gets the input text within both text field boxes.
     * @return type:string Returns the result of the cell phone number and password number with a : separator inbetween.
     */
    function getInput() {
        var celltext = (box_cellfield.acceptableInput) ? box_cellfield.getText() : "NULL";
        var passtext = (box_passfield.acceptableInput) ? box_passfield.getText() : "NULL";
        return celltext + ":" + passtext;
    }

    /**
     * @brief Clears the contents within the text boxes.
     */
    function clear() {
        box_cellfield.clear();
        box_passfield.clear();
    }

    Item {
        anchors {
            fill: parent
            topMargin: 60 * main.vScaleFactor
            leftMargin: 20 * main.hScaleFactor
            rightMargin: 20 * main.hScaleFactor
            bottomMargin: 32 * main.vScaleFactor
        }

        Text {
            id: box_title
            anchors {
                left: parent.left
                top: parent.top
                right: parent.right
            }
            color: "white"
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignTop
            wrapMode: Text.Wrap
            font.pixelSize: 14 * main.vScaleFactor
            text: qsTr("RETURNING USER") + application.emptyString
        }

        StandardTextField {
            id: box_cellfield
            anchors.horizontalCenter: parent.horizontalCenter
            y: 60 * main.vScaleFactor
            width: 270 * main.hScaleFactor
            height: 42 * main.vScaleFactor
            placeholderText: qsTr("CELL PHONE NUMBER") + application.emptyString
            inputMethodHints: Qt.ImhDigitsOnly
            validator: RegExpValidator { regExp: /^\d{3,32}$/ }
        }

        StandardTextField {
            id: box_passfield
            anchors.horizontalCenter: parent.horizontalCenter
            y: 120 * main.vScaleFactor
            width: 270 * main.hScaleFactor
            height: 42 * main.vScaleFactor
            placeholderText: qsTr("PASSWORD") + application.emptyString
            inputMethodHints: Qt.ImhDigitsOnly
            validator: RegExpValidator { regExp: /^\d{4,32}$/ }
            echoMode: TextInput.Password
        }

        StandardButton {
            id: box_button
            anchors {
                bottom: parent.bottom
                horizontalCenter: parent.horizontalCenter
            }
            buttonText: qsTr("SIGN IN") + application.emptyString
            textColor: main.colorFontGreen
            onClicked: {
                box.submitClicked()
                box.clear()
            }

            icon.sourceName: "Components/ArrowRightGreenIcon"
        }
    }
}
