import QtQuick 2.12
import Components 1.0
import Process 0.1
import ".."

StandardPage {
    id: root
    width: main.screenWidth
    height: main.screenHeight

    property string titleText: ""
    title: titleText
    fontDark: true

    image_width: 0.2
    banner_home_button: false
    showBackButton: false

    isSetupWizard: true
    no_bottom_margin: true

    property bool isNucleus: false
    property bool nucleusUpdateAvailable: false
    property bool softwareUpdatesAvailable: checkUpdates.packages.length > 0

    signal setupWizardComplete()
    signal languageSelected(string lang)

    onOpacityChanged: {
        if(opacity) {
            state = "IntroState"
        }
    }

    Component.onCompleted: {
        checkUpdates.start()
    }

    Connections {
        target: contentItem
        ignoreUnknownSignals: true
        onLanguageSelected: {
            languageSelected(lang)
            nextButton.enabled = true
        }
        onItemSelected: {
            nextButton.enabled = true
        }
    }

    TextLabel {
        id: titleLabel
        text: qsTr("SETUP WIZARD") + application.emptyString
        font.pixelSize: 18 * main.vScaleFactor
        anchors {
            top: root.top
            topMargin: 10 * main.vScaleFactor
            left: root.left
            leftMargin: 20 * main.hScaleFactor
        }
    }

    function setCalibrationState() {
        var calIndex = stateLabels.count - 1
        if(machineinfo.productTypeString === "Treadmill") {
            if (!isNucleus) {
                stateLabels.setProperty(calIndex, "state", "InclineCalibrationState");
            } else if (isNucleus && !machinecontrol.isCalibrated) {
                stateLabels.setProperty(calIndex, "state", "InclineCalibrationState");
            }
        } else if (machineinfo.modelString === "TS1000" || machineinfo.modelString === "XS1000") {
            stateLabels.setProperty(calIndex, "state", "StrideCalibrationState");
        }
        else
            stateLabels.setProperty(calIndex, "state", "");

        // Options and Network setup skipped by default
//        stateLabels.setProperty(3, "state", "");
//        stateLabels.setProperty(4, "state", "");
    }

    function toggleSelection(index) {
        var c;
        for (c = 0; c < stateLabels.count; c++) {
            if (c === index)
                stateLabels.setProperty(c, "isSelected", true)
            else
                stateLabels.setProperty(c, "isSelected", false)
        }
    }

    function setDisplay(index) {
        var c;
        for (c = 0; c < stateLabels.count; c++) {
            if (c <= index && stateLabels.get(c).state !== ""){
                stateLabels.setProperty(c, "isDisplayed", true);
            } else {
                stateLabels.setProperty(c, "isDisplayed", false);
            }
        }
    }

    function findIndex() {
        var c;
        for (c = 0; c < stateLabels.count; c++) {
            if (stateLabels.get(c).state === root.state)
                return c;
        }
    }

    function setButtonState() {
        var index;
        index = findIndex();
        toggleSelection(index);
        setDisplay(index);
    }

    ListModel {
        id: stateLabels
        ListElement { label: "LANGUAGE"; state: "LanguageState"; isSelected: false; isDisplayed: false }
        ListElement { label: "PRODUCT ID"; state: "ProductStateNucleus"; isSelected: false; isDisplayed: false }
        ListElement { label: "DATE / TIME"; state: "DateTimeState"; isSelected: false; isDisplayed: false }
        //        ListElement { label: "OPTIONS"; state: "OptionState"; isSelected: false; isDisplayed: false }
        ListElement { label: "NETWORK"; state: "NetworkState"; isSelected: false; isDisplayed: false }
        ListElement { label: "SOFTWARE UPDATE"; state: "SoftwareUpdateState"; isSelected: false; isDisplayed: false }
        ListElement { label: "TV"; state: "TvSetupState"; isSelected: false; isDisplayed: false }
        ListElement { label: "CALIBRATION"; state: "CalibrationState"; isSelected: false; isDisplayed: false }
    }

    Component {
        id: stateButtonDelegate

        StandardButton {
            visible: model.isDisplayed
            selected: model.isSelected
            height: 80 * main.vScaleFactor
            buttonText: qsTranslate("Diagnostics", model.label) + application.emptyString
            label.font.pixelSize: 16 * main.vScaleFactor
            onClicked: {
                if(!selected) {
                    buzzer.buttonPress()
                    toggleSelection(index);
                    setDisplay(index);
                    root.state = model.state;
                }
            }
        }
    }

    ListView {
        id: stateButtons

        width: image_width
        anchors.left: parent.left
        anchors.leftMargin: (image_width * main.screenWidth - 182 * main.hScaleFactor)/2
        anchors.top: parent.top
        anchors.topMargin: 2*titleLabel.height
        anchors.bottom: parent.bottom
        spacing: 25 * main.vScaleFactor
        z: banner.z + 1

        model: stateLabels

        delegate: stateButtonDelegate
    }

    StandardButton {
        id: nextButton

        anchors {
            right: parent.right
            rightMargin: 46
            bottom: parent.bottom
            bottomMargin: 25
        }
        width: 90 * main.hScaleFactor
        height: 73 * main.vScaleFactor
        wideImage: true

        sourceName: "HomeScreen/QuickStartIcon"

        onClicked: changeState()
    }

    function changeState() {
        if(stateLabels.get(stateLabels.count-1).state === "CalibrationState")
            setCalibrationState();

        if(root.state === "IntroState") {
            root.state = "LanguageState"
        }
        else if(root.state === "LanguageState") {
            options.setupWizardCompleted = true
            root.state = "ProductStateNucleus"
        }
        else if (root.state === "ProductStateNucleus") {
            root.state = "DateTimeState"
        }
        else if(root.state === "DateTimeState"){
            contentItem.saveDateTime();

            root.state = "NetworkState";
        }
        else if(root.state === "OptionState")
            root.state = "NetworkState"
        else if(root.state === "NetworkState") {
            if(isNucleus && root.nucleusUpdateAvailable || root.softwareUpdatesAvailable)
                root.state = "SoftwareUpdateState"
            else
                root.state = "TvSetupState"
        } else if (root.state === "SoftwareUpdateState")
            root.state = "TvSetupState"
        else if (root.state === "TvSetupState") {
            contentItem.doSave();
            options.setupWizardCompleted = true

            if (isNucleus) {
                root.state = "SetupCompleteState";
            }

            if(machineinfo.productTypeString === "Treadmill") {
                if (!isNucleus) {
                    root.state = "InclineCalibrationState";
                } else if (isNucleus && !machinecontrol.isCalibrated) {
                    root.state = "InclineCalibrationState";
                } else {
                    root.state = "SetupCompleteState";
                }
            } else if (machineinfo.modelString === "TS1000" || machineinfo.modelString === "XS1000") {
                root.state = "StrideCalibrationState";
            } else {
                root.state = "SetupCompleteState";
            }
        } else if(root.state === "InclineCalibrationState") {
            root.state = "SetupCompleteState";
        } else if (root.state === "StrideCalibrationState") {
            root.state = "SetupCompleteState";
        } else if(root.state === "SetupCompleteState") {
            setupWizardComplete();
        }

        setButtonState();
    }

    states: [
        State {
            name: "IntroState"

            PropertyChanges { target: root; contentSource: "SetupWizardIntro.qml" }
            PropertyChanges { target: nextButton; enabled: true }
            PropertyChanges { target: root; titleText: "" }
            PropertyChanges { target: root; showTitle: false}
        },
        State {
            name: "LanguageState"

            PropertyChanges { target: root; contentSource: "SetupWizardLanguageUnits.qml" }
            PropertyChanges { target: nextButton; enabled: false }
            PropertyChanges { target: root; titleText: qsTr("SELECT DEFAULT LANGUAGE AND UNITS") }
            PropertyChanges { target: root; showTitle: true}
        },
        State {
            name: "ProductStateNucleus"

            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsSetupNucleus.qml" }
            PropertyChanges { target: nextButton; enabled: true }
            PropertyChanges { target: root; titleText: ""}
            PropertyChanges { target: root; showTitle: false}
        },
//        State {
//            name: "OptionState"

//            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsOption.qml" }
//            PropertyChanges { target: nextButton; enabled: true }
//        },
        State {
            name: "NetworkState"

            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsNetwork.qml" }
            PropertyChanges { target: nextButton; enabled: true }
            PropertyChanges { target: root; titleText: ""}
            PropertyChanges { target: root; showTitle: false}
        },
        State {
            name: "DateTimeState"

            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsClockSetup.qml" }
            PropertyChanges { target: nextButton; enabled: true }
            PropertyChanges { target: root; titleText: "" }
            PropertyChanges { target: root; showTitle: false}
        },
        State {
            name: "SoftwareUpdateState"

            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsUpdate.qml" }
            PropertyChanges { target: nextButton; enabled:root.nucleusUpdateAvailable || root.softwareUpdatesAvailable}
            PropertyChanges { target: root; titleText: "" }
            PropertyChanges { target: root; showTitle: false}
        },
        State {
            name: "TvSetupState"

            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsTVSetup.qml" }
            PropertyChanges { target: nextButton; enabled: !contentItem.scanning }
            PropertyChanges { target: root; titleText: "" }
            PropertyChanges { target: root; showTitle: false}
        },
        State {
            name: "InclineCalibrationState"

            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsTreadmillCalibration.qml" }
            PropertyChanges { target: nextButton; enabled: true }
            PropertyChanges { target: root; titleText: "" }
            PropertyChanges { target: root; showTitle: false}
        },
        State {
            name: "StrideCalibrationState"

            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsStrideEllipticalCalibration.qml" }
            PropertyChanges { target: nextButton; enabled: true }
            PropertyChanges { target: root; titleText: "" }
            PropertyChanges { target: root; showTitle: false}
        },
        State {
            name: "InclineCalibrationTestState"

            PropertyChanges { target: root; contentSource: "../Diagnostics/DiagnosticsSummary.qml" }
            PropertyChanges { target: nextButton; enabled: true }
            PropertyChanges { target: root; titleText: qsTr("SUMMARY") }
            PropertyChanges { target: root; showTitle: true}
        },
        State {
            name: "SetupCompleteState"

            PropertyChanges { target: root; contentSource: "SetupWizardComplete.qml" }
            PropertyChanges { target: nextButton; enabled: true }
            PropertyChanges { target: root; titleText: "" }
            PropertyChanges { target: root; showTitle: false}
        }
    ]

    // The following process gathers a list of packages that may be updated
    Process {
        id: checkUpdates
        property string packages

        program: "bash"
        arguments: [ "scripts/list_upgrades.sh" ]
        onReadyReadStandardOutput: {
            packages += readAllStandardOutput()
        }
        onStarted: {
            packages = ""
            nucleusfirmwareloader.findHexFile()
            console.log("checking updates")
        }
        onFinished: console.log("packages upgradable:" + packages)
    }
}
