import QtQuick 2.12
import ".."
import Components 1.0

ModalDialog {
    id: root

    height: 1.5 * main.heightDialog
    property string iconsSource: "qml/graphics/ui3_0/Diagnostics/channelIcons"

    signal iconSelected(string iconSource)
    signal exitSelected()

    function loadIconList() {
        var images = storageMgr.listFiles(root.iconsSource, ["*.png", "*.jpg"])
        iconListModel.clear()
        for (var i = 0; i < images.length; i++) {
            if (images[i].indexOf("tv_default") >= 0){
                iconListModel.insert(0, {imageSource: images[i]})
                continue
            }
            iconListModel.append({ imageSource: images[i] })
        }
        gridView.currentIndex = 0
    }

    Component.onCompleted: loadIconList()

    onVisibleChanged: {
        if (!visible) {
            gridView.currentIndex = 0
            gridView.contentY = 0
        }
    }

    ListModel {
        id: iconListModel

        ListElement { imageSource: "fx.png" }
    }

    Item {
        z: root.z + 1
        anchors.fill: parent
//        clip: true

        CloseButton {
            id: closeButton

            anchors.right: parent.right
            anchors.top: parent.top
            anchors.topMargin: 10 * main.vScaleFactor
            anchors.rightMargin: 10 * main.hScaleFactor
            onClicked: {
                hide()
                exitSelected()
            }
        }

        Item {
            id: dataArrows

            width: 40 * main.hScaleFactor
            height: parent.height

            anchors { top: parent.top; right: parent.right ; rightMargin: -20 * main.hScaleFactor }
            ScalableImage {
                id: dataUpArrow
                visible: !gridView.atYBeginning
                rotation: 90
                anchors { top: parent.top; topMargin: 35 * main.vScaleFactor; right: parent.right; rightMargin: 25 * main.hScaleFactor }
                sourceName: "Swipe_Arrows_Left";
            }
            ScalableImage {
                id: dataDownArrow
                visible: !gridView.atYEnd
                rotation: 90
                anchors { bottom: parent.bottom; bottomMargin: 35 * main.vScaleFactor; right: parent.right; rightMargin: 25 * main.hScaleFactor }
                sourceName: "Swipe_Arrows_Right";
            }
        }

        GridView {
            id: gridView

            width: parent.width; height: parent.height
            anchors.fill: parent
            anchors {
                leftMargin: 25 * main.hScaleFactor
                rightMargin: 45 * main.hScaleFactor
                topMargin: 25 * main.vScaleFactor
                bottomMargin: 25 * main.vScaleFactor
            }

            clip: true
            cellWidth: 110 * main.hScaleFactor
            cellHeight: 110 * main.vScaleFactor
            model: iconListModel
            delegate: Component {
                Rectangle {
                    function selectIcon(){
                        root.iconSelected(model.imageSource)
                    }
                    color: "white"
                    width: 100 * main.hScaleFactor
                    height: 100 * main.hScaleFactor
                    ScalableImage {
                        anchors.fill: parent
                        sourceName: "Diagnostics/channelIcons/" + model.imageSource.replace(/\.[^/.]+$/, "");
                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                selectIcon()
                            }
                            Rectangle {
                                anchors.fill: parent
                                color: main.colorGenericStimulusArea
                                opacity: 0.3
                                visible: main.showStimulusArea
                            }
                        }
                    }

                }
            }
        }
    }

}
