import QtQuick 2.12
import "string_format.js" as Format
import Components 1.0
import ".."

Item {
    id: root
    property variant c360program: workoutProgram.c360Program
    property int segmentTime: c360program.phaseTimeRemaining
    property int phase: c360program.currentPhase

    signal phaseTitleChanged(string title)
    anchors.fill: parent

    onPhaseChanged: {
        var text = qsTr(c360Modes_Elliptical.title(phase))
        root.phaseTitleChanged(text)
    }

    Component {
        id: phaseImages

        Item {
            id: c360PhaseView
            property int imageIndex: viewObject.imageNum

            onImageIndexChanged: {
                photo.source = (c360Modes_Elliptical.imagePath() + images.get(imageIndex).source)
            }
            anchors.fill: parent

            ScrollingText {
                id: titleText

                boundsBehavior: Flickable.StopAtBounds
                text: c360program.coolDown ? "" : qsTr(viewObject.model.title(index)) + application.emptyString
                font.pixelSize: 40 * main.vScaleFactor
                anchors {
                    horizontalCenter: parent.horizontalCenter
                    top: parent.top
                    topMargin: 20 * main.vScaleFactor
                }
//                    wrapMode: Text.Wrap
//                    lineHeight: 0.7
                width: parent.width
//                    elide: Text.ElideRight
            }

            Image { // Not changed due to .jpgs - could change prefix
                id: photo
                anchors {
                    top: titleText.bottom
                    topMargin: 40 * main.vScaleFactor
                    left: parent.left
                    leftMargin: 50 * main.hScaleFactor
                    bottom: parent.bottom;
                    bottomMargin: 80 * main.vScaleFactor
                }
                source: (c360Modes_Elliptical.imagePath() + images.get(0).source)

                fillMode: Image.PreserveAspectFit
                smooth: true
            }

            TextLabel {
                id: instText
                text: qsTr(viewObject.model.instructions(index)) + application.emptyString
                anchors {
                    verticalCenter: photo.verticalCenter
                    left: photo.right
                    leftMargin: 10 * main.hScaleFactor
                }
            }

            ScalableImage {
                id: motionMan
                anchors {
                    top: photo.top
                    bottom: photo.bottom
                    right: parent.right
                    rightMargin: 100 * main.hScaleFactor

                }
                fillMode: Image.PreserveAspectFit
                smooth: true

                sourceName: "c360/MotionMan/" + icon;
            }

            TextLabel {
                id: segTimeLabel
                anchors {
                    bottom: photo.bottom
                    left: photo.right
                    leftMargin: 10  * main.hScaleFactor
                }
                text: qsTr("SEGMENT TIME REMAINING") + application.emptyString
                font.pixelSize: 12 * main.vScaleFactor
            }

            NumberLabel {
                id: segTimeIndicator
                anchors {
                    left: segTimeLabel.right
                    leftMargin: 10  * main.hScaleFactor
                    bottom: segTimeLabel.bottom
                }
                text: Format.formatTimeString(root.segmentTime)
                font.pixelSize: 30 * main.vScaleFactor
            }
        }
    }

    PathView {
        id: viewObject
        property int imageNum: 0

        preferredHighlightBegin: 0.5
        preferredHighlightEnd: 0.5
        pathItemCount: 1
        anchors.fill: parent
        interactive: false
        currentIndex: root.phase

        model: c360Modes_Elliptical
        delegate: phaseImages
        clip: true
        path: Path {
            startX: viewObject.width
            startY: viewObject.height / 2
            PathLine {
                x: viewObject.width
                y: viewObject.height / 2
            }
        }

        // timer to switch photo images between full and close-up
        Timer {
            id: imageTimer
            running: true
            repeat: true
            interval: 6000
            onTriggered: {
                viewObject.imageNum = (viewObject.imageNum ? 0 : 1)
            }
        }
    }
}
