import QtQuick 2.12
import Components 1.0
import "../"
import WorkoutManager 1.0
import WorkoutRunData 1.0

Item {
    id: root

    property int elapsedTime: aWorkoutRunData.workoutTime + aWorkoutRunData.coolDownTimeproperty
    property bool isTreadmill: machineinfo.productTypeString === "Treadmill"

    property variant model1
    property variant model2

    // set height of bar to draw based on the level or magnitude property of the profile segment, as appropriate
    property int borderWidth: 2 * main.scaleFactor
    property real yPositionOffsetPercent: 0.05 // Adds additional space between bottom of bar and bottom of container, if needed
    property int yPosition: (0.1 + yPositionOffsetPercent)*root.height
    property int topYPosition: 0.95*root.height
    property int maxBarHeight: root.topYPosition - root.yPosition // - root.borderWidth + heightDifferential - baseOffset

    property bool hasNegativeValues: machineinfo.treadmillInclineMinPercentGrade < 0.0
    property int numNegativeValues: hasNegativeValues ? machineinfo.inclineTable.indexOf(0.0) +1 : 0

    clip: true

    property bool isMenuDisplayed: false
    signal action()

    Rectangle { id: backgroundReference
        anchors.left: parent.left
        anchors.leftMargin: 50 * main.hScaleFactor
        anchors.bottom: parent.bottom
        width: root.width - 100 * main.hScaleFactor
        height: root.height
        border.color: "grey"
        border.width: 2
        color: "transparent"
        visible: false
    }

    Item {
        id: lineGraphsView

        width: root.width - 72 * main.hScaleFactor
        height: root.height
        anchors {
            left: root.left
            leftMargin: 70 * main.hScaleFactor
            top: root.top
            topMargin: root.yPositionOffsetPercent*root.height
        }

        property int timerInterval: 1000
        property bool isTarget1: false
        property bool isTarget2: false
        property int displayTime: 600 // 10 minutes on screen at a time
        property bool isScrollLock: options.scrollLock
        property int scrollTime: isScrollLock ? timerInterval/1000 : displayTime/2
        property int scrollLockTime: (isScrollLock ? 0.8 : 1 )*displayTime
        property double timeToPixels: lineGraphsView.width/displayTime
        property int endX: displayTime * timeToPixels
        property int startT: 0
        property int startX: startT * timeToPixels
        property bool scrollFlag: false

        property bool gridPainted: false

        property int hr_runningBPM: workoutManager.getRunningBPM()
        property int hr_approachingBPM: workoutManager.getApproachingBPM()
        property int hr_beginningBPM: workoutManager.getBeginningBPM()
        property int hr_exceedingBPM: workoutManager.getExceedingBPM()
        property bool isHRC: false
        property int  maxHeartRate: isHRC ? aWorkoutRunData.targetHeartRate + 2*hr_exceedingBPM : 251 // Top value for graph
        property int  minHeartRate: 20 // Bottom value for graph

        property int minInclineGrade: machineinfo.treadmillInclineMinPercentGrade
        property int maxInclineGrade: options.maxInclineRegulatorEnabled ? options.maxInclineRegulatorValue : machineinfo.treadmillInclineMaxPercentGrade

        property int maxY1: computeMaxY1()
        property int maxY2: computeMaxY2()
        property int minY1: computeMinY1()
        property int minY2: computeMinY2()

        clip: true

        function getIsTargetType(segOption) {
            switch (segOption) {
                case WorkoutProfile.UNDEFINED_SEG_OPTION:
                case WorkoutProfile.FIXED_SEG_OPTION:
                case WorkoutProfile.CONSTANT_SEG_OPTION:
                    return false;
                case WorkoutProfile.VARIED_LEVELS_SEG_OPTION:
                case WorkoutProfile.INTERVALS_SEG_OPTION:
                case WorkoutProfile.CUSTOM_SEG_OPTION:
                case WorkoutProfile.CALCULATED_SEG_OPTION:
                case WorkoutProfile.EGYM_SEG_OPTION:
                   return true;
            }
        }

        function computeY(target, max) {
            var percent = target / max
            var y = percent * root.maxBarHeight + root.yPosition
            return root.topYPosition - y // 0 is top left
        }

        function computeYinRange(target, max, min) {
            var percent = (target - min) / (max - min)
            var y = percent * root.maxBarHeight + root.yPosition
            return root.topYPosition - y // 0 is top left
        }
 
        function computeMaxY1() {
            if(isTreadmill) {
                return (lineGraphsView.maxInclineGrade + 5)
            } else if (workoutProgram.displayDataVars.wattsAvailable) {
                return (lineGraphsView.isTarget1 ? root.model1.magnitudeRangeMax + 1 : workoutProgram.brakeWattsTable[workoutProgram.brakeWattsTable.length-1])
            } else {
                return 1
            }
        }

        function computeMaxY2() {
            if(isTreadmill) {
                return (root.model2.magnitudeRangeMax + 1)
            } else if (main.isStepmill) {
                return (workoutManager.workoutProgram.maxSpeed + 1)
            } else {
                return (workoutProgram.displayDataVars.paceSetterAvailable ? root.model2.magnitudeRangeMax + 1: 200)
            }
        }

        function computeMinY1() {
            if(isTreadmill) {
                return (lineGraphsView.minInclineGrade - 5)
            } else {
                return 0
            }
        }

        function computeMinY2() {
            return 0
        }

        Canvas {
            id: gridGraph
            anchors.fill: parent

            function paintGrid() {
                if(!gridGraph.available || lineGraphsView.gridPainted)
                    return

                var ctx = gridGraph.getContext("2d")
                ctx.clearRect(0, 0, gridGraph.width, gridGraph.height)

                ctx.strokeStyle = Qt.rgba(0.225, 0.225, 0.225, 1.0)
                ctx.lineWidth = 2*main.scaleFactor

                var x = 0
                var maxX = lineGraphsView.contentWidth
                var deltaX = 60 * lineGraphsView.timeToPixels
                var y =  0
                var maxY = root.maxBarHeight + y
                var deltaY = maxY/10

                while (x < maxX){
                    ctx.beginPath()
                    ctx.moveTo(x, y)
                    ctx.lineTo(x, maxY)
                    ctx.stroke()
                    x += deltaX
                }

                while (y < maxY){
                    ctx.beginPath()
                    ctx.moveTo(0,y)
                    ctx.lineTo(maxX,y)
                    ctx.stroke()
                    y += deltaY
                }

                lineGraphsView.gridPainted = true
            }

            onPaint: {
//                paintGrid()
            }
        }

        Canvas {
            id: targetGraphs
            anchors.fill: parent

            property bool hrc_inRestPhase: false
            property bool hrFirstCall: true

            property var inRestPts: []
            property var xPts: []
            property var targetPts: []
            property var pacePts: []
            property var modePts: []
            property bool redrawFlag: false
            property bool firstCooldownFlag: true

            function scrollPoints() {
                var i=0
                while (i < xPts.length && xPts[i] < lineGraphsView.startX) {
                    i++
                }

 //               console.log("Scroll Target skipping " + i + " points of " + xPts.length)
                if(i > 1) {
                    xPts.splice(0,i-1)
                    inRestPts.splice(0,i-1)
                    targetPts.splice(0,i-1)
                    pacePts.splice(0,i-1)
                    modePts.splice(0,i-1)
                }

                redrawFlag = true
            }

            function storePoints(x, target, pace, mode, inRest) {
                xPts.push(x)
                targetPts.push(target)
                pacePts.push(pace)
                modePts.push(mode)
                inRestPts.push(inRest)
//                console.log("Saving Target Graph Point (" + xPts.length + "): [" + x + ", " + target + ", " + pace + ", " + mode + ", " + inRest + "]")
            }

            function paintTarget(model, strokeColor, modelMax, modelMin) {
                if(!targetGraphs.available)
                    return

                var deltaX = model.segmentDuration * lineGraphsView.timeToPixels
                var y = root.yPositionOffsetPercent*root.height

                var ctx = targetGraphs.getContext("2d")

                ctx.strokeStyle = strokeColor
                ctx.lineWidth = 4*main.scaleFactor

                // loop through the profile and paint the target values
                var i
                var startI = Math.floor(lineGraphsView.startX/deltaX)
                var x = startI * deltaX - lineGraphsView.startX
                ctx.beginPath()
                for (i=startI; i < model.count; i++) {

                    var target = model.getProperty(i, "segmentMagnitude")
                    y = lineGraphsView.computeYinRange(target, modelMax, modelMin)

                    ctx.lineTo(x,y)
                    ctx.lineTo(x+deltaX,y)
                    ctx.stroke()
                    x += deltaX

                    if(x > lineGraphsView.endX)
                        break
                }
            }

            function paintLine(minX, maxX, pace, strokeColor, modelMax, modelMin) {
                if(!targetGraphs.available)
                    return

                var ctx = targetGraphs.getContext("2d")

                ctx.strokeStyle = strokeColor
                ctx.lineWidth = 4*main.scaleFactor

                // paint the target line
                ctx.beginPath()
                var lineY = lineGraphsView.computeYinRange(pace, modelMax, modelMin)
                ctx.lineTo(minX-lineGraphsView.startX,lineY)
                ctx.lineTo(maxX-lineGraphsView.startX,lineY)
                ctx.stroke()
            }

            function drawTargetRegion(ctx, minX, deltaX, y1, y2, y3, inWarmup) {
//                console.log("drawTargetRegion " + minX + "-" + lineGraphsView.startX + ", " + deltaX + ", " + inWarmup)
                // Target or Approaching Region
                ctx.beginPath()
                ctx.fillStyle = Qt.rgba(200/255,80/255,44/255, inWarmup ? 0.3 : 0.8) // #3CB4F4
                ctx.fillRect(minX-lineGraphsView.startX, y1, deltaX, y2-y1)

                // Not to Exceed or Beginning Region
                ctx.beginPath()
                ctx.fillStyle = Qt.rgba(180/255,60/255,34/255, 0.5)
                ctx.fillRect(minX-lineGraphsView.startX, y2, deltaX, y3-y2)
            }

            function paintTargetHR() {
                if(!targetGraphs.available)
                    return

                if (!lineGraphsView.isHRC && !workoutManager.cruiseControlEnabled)
                    return;

                var ctx = targetGraphs.getContext("2d")

                var x = aWorkoutRunData.totalTime * lineGraphsView.timeToPixels
                var exY = lineGraphsView.computeYinRange(aWorkoutRunData.targetHeartRate+lineGraphsView.hr_exceedingBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)

                var maxHR = lineGraphsView.computeYinRange(aWorkoutRunData.targetHeartRate+lineGraphsView.hr_runningBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                var minHR = lineGraphsView.computeYinRange(aWorkoutRunData.targetHeartRate-lineGraphsView.hr_runningBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)

                var avgPace = 0
                if (!isTreadmill && !main.isStepmill && aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE)
                    avgPace = aWorkoutRunData.totalAveragePace

                storePoints(x, aWorkoutRunData.targetHeartRate, avgPace, aWorkoutRunData.mode, targetGraphs.hrc_inRestPhase)
/*
                if (aWorkoutRunData.mode === WorkoutRunData.WARM_UP_MODE) {
                    var minY = lineGraphsView.computeYinRange(aWorkoutRunData.targetHeartRate-lineGraphsView.hr_approachingBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                    var midY = lineGraphsView.computeYinRange(aWorkoutRunData.targetHeartRate-lineGraphsView.hr_beginningBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)

                    drawTargetRegion(ctx, lineGraphsView.startX, lineGraphsView.endX, minY, midY, exY, true)

                } else if (aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE && targetGraphs.hrFirstCall) {

                    drawTargetRegion(ctx, x, lineGraphsView.endX, minHR, maxHR, exY, false)

                    if(avgPace > 0)
                        paintLine(x, lineGraphsView.endX+lineGraphsView.startX, avgPace, Qt.rgba(0/255,120/255,147/255, 1.0), lineGraphsView.maxY2, lineGraphsView.minY2);

                    if(targetGraphs.hrFirstCall) {
                        targetGraphs.hrFirstCall = false
                    }
                } else if (aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE &&
                           workoutProgram.isHRCCustomProgram && targetGraphs.hrc_inRestPhase) {
                    var deltaX = workoutProgram.restTime * lineGraphsView.timeToPixels
                    drawTargetRegion(ctx, x, deltaX, minHR, maxHR, exY, false)
                }
                */
            }
/*
            function clearGraph() {
                if(!targetGraphs.available)
                    return

                var ctx = targetGraphs.getContext("2d")

                if (aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE &&
                        (lineGraphsView.isTarget1 || lineGraphsView.isTarget2)) {
//                    console.log("Clearing Graph " + currMin_X + ", " + (targetGraphs.width - currMin_X))
                    ctx.clearRect(0, 0, targetGraphs.width, targetGraphs.height)
                } else if ((aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE && (targetGraphs.hrc_inRestPhase || targetGraphs.hrFirstCall)) || aWorkoutRunData.mode === WorkoutRunData.COOL_DOWN_MODE){
                    // clear rest interval or rest of graph
                    var restInterval = workoutProgram.restTime
                    var x = aWorkoutRunData.totalTime * lineGraphsView.timeToPixels - lineGraphsView.startX
                    var w = targetGraphs.hrc_inRestPhase ? restInterval * lineGraphsView.timeToPixels : (targetGraphs.width - x)

                    ctx.clearRect(x, 0, w, targetGraphs.height)
                    console.log("Clearing Graph " + targetGraphs.hrc_inRestPhase + ", " + x + ", " + w)
                }
            }
*/
            function redrawHRGraphs() {
//                console.log("redrawHRGraphs " + redrawFlag)
                if(!targetGraphs.available || !redrawFlag)
                    return

                if (!lineGraphsView.isHRC && !workoutManager.cruiseControlEnabled)
                    return;

                var ctx = targetGraphs.getContext("2d")
                ctx.clearRect(0, 0, targetGraphs.width, targetGraphs.height)

                var i
                for(i=0; i < targetGraphs.xPts.length; i++) {
                    if(workoutManager.cruiseControlEnabled && i===0)
                        continue;

                    var target = targetGraphs.targetPts[i]
                    var exY = lineGraphsView.computeYinRange(target+lineGraphsView.hr_exceedingBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)

                    if(targetGraphs.modePts[i] === WorkoutRunData.WARM_UP_MODE) {
                        var minY = lineGraphsView.computeYinRange(target-lineGraphsView.hr_approachingBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                        var midY = lineGraphsView.computeYinRange(target-lineGraphsView.hr_beginningBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                        var warmX = (i === targetGraphs.xPts.length-1) ? lineGraphsView.endX+lineGraphsView.startX : xPts[i+1]

                        drawTargetRegion(ctx, xPts[i], warmX-xPts[i], minY, midY, exY, true)
                    } else if(targetGraphs.modePts[i] !== WorkoutRunData.COOL_DOWN_MODE) {
                        var maxHR = lineGraphsView.computeYinRange(target+lineGraphsView.hr_runningBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                        var minHR = lineGraphsView.computeYinRange(target-lineGraphsView.hr_runningBPM, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                        var x = (i === targetGraphs.xPts.length-1) ? lineGraphsView.endX+lineGraphsView.startX : xPts[i+1]

                        if(!inRestPts[i] || workoutProgram.isHRCCustomProgram) {
                            drawTargetRegion(ctx, xPts[i], x-xPts[i], minHR, maxHR, exY, false)

                            if(pacePts[i] > 0) {
                                paintLine(xPts[i], x, pacePts[i], Qt.rgba(0/255,120/255,147/255, 1.0), lineGraphsView.maxY2, lineGraphsView.minY2);
                                console.log("Drawing Pace Points " + xPts[i] + ", " + (x) + ", " + pacePts[i])
                            }
                        } else if (inRestPts[i] && i === targetGraphs.xPts.length-1) {
                            // Remove last rest segment if it is repeated
                            var restX = xPts[i]
                            if(inRestPts[i-1]) {
                                restX = xPts[i-1]
                                xPts.pop()
                                targetPts.pop()
                                modePts.pop()
                                inRestPts.pop()
                            }

                            // Draw next work segment
                            restX = restX + workoutProgram.restTime * lineGraphsView.timeToPixels
                            drawTargetRegion(ctx, restX, x-restX, minHR, maxHR, exY, false)

                            if(pacePts[i] > 0) {
                                paintLine(restX, x, pacePts[i], Qt.rgba(0/255,120/255,147/255, 1.0), lineGraphsView.maxY2, lineGraphsView.minY2);
//                                console.log("Drawing Pace Points " + xPts[i] + ", " + (x) + ", " + pacePts[i])
                            }
                        }
                    }
                }

                redrawFlag = false
            }

/*
            function paintGrid() {
                if(!targetGraphs.available || lineGraphsView.gridPainted)
                    return

                var ctx = targetGraphs.getContext("2d")

                ctx.strokeStyle = Qt.rgba(0.225, 0.225, 0.225, 1.0)
                ctx.lineWidth = 2*main.scaleFactor

                var deltaX = 60 * lineGraphsView.timeToPixels
                var y =  0
                var maxY = root.maxBarHeight + y
                var deltaY = maxY/10

                // First section may be a partial minute
                var nextX = Math.ceil(aWorkoutRunData.totalTime/60) * 60 * lineGraphsView.timeToPixels
                var x = nextX+deltaX
                var maxX = targetGraphs.hrc_inRestPhase ? (60 * lineGraphsView.timeToPixels + x) : lineGraphsView.contentWidth

                ctx.beginPath()
                ctx.moveTo(nextX, y)
                ctx.lineTo(nextX, maxY)
                ctx.stroke()

                console.log("paintGrid " + x + ", " + maxX + ", " + y + ", " + maxY)
                while (x < maxX){
                    ctx.beginPath()
                    ctx.moveTo(x, y)
                    ctx.lineTo(x, maxY)
                    ctx.stroke()
                    x += deltaX
                }

                x = aWorkoutRunData.totalTime * lineGraphsView.timeToPixels
                while (y < maxY){
                    ctx.beginPath()
                    ctx.moveTo(x,y)
                    ctx.lineTo(maxX,y)
                    ctx.stroke()
                    y += deltaY
                }

                lineGraphsView.gridPainted = true
            }
*/
            onPaint: {
//                clearGraph()

                if(!targetGraphs.available)
                    return

                if(lineGraphsView.scrollFlag)
                    scrollPoints()

                if (lineGraphsView.isTarget1 || lineGraphsView.isTarget2){
                    var ctx = targetGraphs.getContext("2d")
                    ctx.clearRect(0, 0, targetGraphs.width, targetGraphs.height)
                }

                if (lineGraphsView.isTarget1)
                    paintTarget(root.model1, Qt.rgba(43/255,121/255,0/255, 1.0), lineGraphsView.maxY1, lineGraphsView.minY1)

                if (lineGraphsView.isTarget2)
                    paintTarget(root.model2, Qt.rgba(0/255,120/255,147/255, 1.0), lineGraphsView.maxY2, lineGraphsView.minY2)

                if(aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE || aWorkoutRunData.mode === WorkoutRunData.WARM_UP_MODE) {
                    if(aWorkoutRunData.targetHeartRate !== targetPts[targetPts.length-1] ||
                       aWorkoutRunData.mode !== modePts[modePts.length-1] ||
                       targetGraphs.hrc_inRestPhase !== inRestPts[inRestPts.length-1]) {
                        paintTargetHR()
                        redrawFlag = true
                    }
                } else if(aWorkoutRunData.mode === WorkoutRunData.COOL_DOWN_MODE) {
                    // store final data point
                    if(firstCooldownFlag) {
                        paintTargetHR()
                        redrawFlag = true
                    }

                    firstCooldownFlag = false
                }

                redrawHRGraphs()
            }
        }

        Canvas {
            id: lineGraphs
            anchors.fill: parent

            property var xPts: []
            property var y1Pts: []
            property var y2Pts: []
            property var hrPts: []

            property int lastHR_Y: 0
            property int lastPace_Y: 0
            property int last_Y1: 0
            property double lastX: 0
            property double currTime: aWorkoutRunData.totalTime
            property int currX: aWorkoutRunData.totalTime * lineGraphsView.timeToPixels

            property var y1Color: Qt.rgba(63/255,221/255,0/255, 1.0)
            property var y2Color: Qt.rgba(60/255,180/255,244/255, 1.0)
            property var hrColor: Qt.rgba(0.925, 0.110, 0.141, 1.0)
            property bool redrawFlag: false

            function scrollPoints() {
                var i=0
                while (i<xPts.length && xPts[i] < lineGraphsView.startX) {
                    i++
                }

                if(i>0){
//                    console.log("Scroll Line skipping " + i + " points of " + xPts.length)
                    xPts.splice(0,i)
                    y1Pts.splice(0,i)
                    y2Pts.splice(0,i)
                    hrPts.splice(0,i)
                }

                redrawFlag = true
            }

            function storePoints(x,y1,y2,hr) {
                xPts.push(x)
                y1Pts.push(y1)
                y2Pts.push(y2)
                hrPts.push(hr)
            }

            function drawSeg(ctx, lastX, currX, lastY, currY, strokeColor) {
                ctx.strokeStyle = strokeColor
                ctx.lineWidth = 2*main.scaleFactor

                ctx.beginPath()
                ctx.moveTo(Math.round(lastX-lineGraphsView.startX), Math.round(lastY))
                ctx.lineTo(Math.round(currX-lineGraphsView.startX), Math.round(currY))
                ctx.stroke()
            }

            function paintNextSeg() {
                if(!lineGraphs.available)
                    return

                var ctx = lineGraphs.getContext("2d")

                // Heart Rate
                if(heartRateManager.heartRateAvailable) {
                    var hr_y = lineGraphsView.computeYinRange(heartRateManager.heartRate, lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                    if(lastHR_Y === 0)
                        lastHR_Y = hr_y

                    if(root.visible) drawSeg(ctx, lastX, currX, lastHR_Y, hr_y, hrColor)

                    lastHR_Y = hr_y
                }

                // Speed/Pace
                var pace_y = 0
                if(isTreadmill) {
                    pace_y = lineGraphsView.computeY(machinecontrol.treadmillSpeedActual, lineGraphsView.maxY2)
                } else if (main.isStepmill) {
                    pace_y = lineGraphsView.computeY(machinecontrol.stepmillSpeedActual, lineGraphsView.maxY2)
                } else {
                    pace_y = lineGraphsView.computeY(aWorkoutRunData.rpm, lineGraphsView.maxY2)
                }

                if(lastPace_Y === 0)
                    lastPace_Y = pace_y

                if(root.visible) drawSeg(ctx, lastX, currX, lastPace_Y, pace_y, y2Color)

                lastPace_Y = pace_y

                // Incline
                if(isTreadmill) {
                    var incline_y = lineGraphsView.computeYinRange(machinecontrol.treadmillInclinePercentGradeActual, lineGraphsView.maxY1, lineGraphsView.minY1)
                    if(last_Y1 === 0)
                        last_Y1 = incline_y

                    if(root.visible) drawSeg(ctx, lastX, currX, last_Y1, incline_y, y1Color)

                    last_Y1 = incline_y
                } else if (workoutProgram.displayDataVars.wattsAvailable) {
                    // Watts
                    var watts_y = lineGraphsView.computeY(aWorkoutRunData.watts, lineGraphsView.maxY1)
                    if(last_Y1 === 0)
                        last_Y1 = watts_y

                    if(root.visible) drawSeg(ctx, lastX, currX, last_Y1, watts_y, y1Color)

                    last_Y1 = watts_y
                }

                lastX = currX
                storePoints(lastX, last_Y1, lastPace_Y, heartRateManager.heartRate)
            }

            function redrawGraphs() {
                if(!lineGraphs.available || !redrawFlag || !root.visible)
                    return

                var ctx = lineGraphs.getContext("2d")
                ctx.clearRect(0, 0, lineGraphs.width, lineGraphs.height)

                var i
                for(i=1; i<xPts.length; i++) {
                    if(y1Pts[i]>0)
                        drawSeg(ctx, xPts[i-1], xPts[i], y1Pts[i-1], y1Pts[i], y1Color)
                    if(y2Pts[i]>0)
                        drawSeg(ctx, xPts[i-1], xPts[i], y2Pts[i-1], y2Pts[i], y2Color)
                    if(hrPts[i]>0 && hrPts[i-1]>0) {
                        var hr_y1 = lineGraphsView.computeYinRange(hrPts[i-1], lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                        var hr_y2 = lineGraphsView.computeYinRange(hrPts[i], lineGraphsView.maxHeartRate, lineGraphsView.minHeartRate)
                        drawSeg(ctx, xPts[i-1], xPts[i], hr_y1, hr_y2, hrColor)
                    }
                }

                redrawFlag = false
            }

            onPaint: {
                if(lineGraphsView.scrollFlag)
                    scrollPoints()

                paintNextSeg()
                redrawGraphs()
            }
        }

        Connections {
            target: root.model1
            onDataChanged: {
                if(lineGraphsView.isTarget1 && aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE) {
                    targetGraphs.requestPaint()
                }
            }
        }

        Connections {
            target: root.model2
            onDataChanged: {
                if(lineGraphsView.isTarget2 && aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE) {
                    targetGraphs.requestPaint()
                }
            }
        }

        Connections {
            target: workoutManager

            onWorkoutModeChanged: {
                if(aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE) {
                    lineGraphsView.gridPainted = false
                    lineGraphsView.isTarget1 = lineGraphsView.getIsTargetType(workoutProgram.profile1.segmentOption)
                    lineGraphsView.isTarget2 = lineGraphsView.getIsTargetType(workoutProgram.profile2.segmentOption)
//                    gridGraph.requestPaint()
//                    lineGraphsView.warmupWidth = Math.max(aWorkoutRunData.warmUpTime, lineGraphsView.warmupTime) * lineGraphsView.timeToPixels
                    targetGraphs.requestPaint()
                }

                if(aWorkoutRunData.mode === WorkoutRunData.COOL_DOWN_MODE || aWorkoutRunData.mode === WorkoutRunData.WARM_UP_MODE) {
                    targetGraphs.hrc_inRestPhase = false
                    targetGraphs.requestPaint()
                }
            }

            onHrcRestPhaseChanged: {
                targetGraphs.hrc_inRestPhase = workoutManager.hrcRestPhase
                targetGraphs.requestPaint()
            }
        }

        Connections {
            target: workoutProgram
            onIsHRCProgramChanged: {
                if(!workoutProgram.isHRCProgram) {
                    lineGraphsView.isHRC = true
                    targetGraphs.hrc_inRestPhase = false
                }
            }
            onTargetHRChanged: {
                targetGraphs.redrawFlag=true
                targetGraphs.requestPaint()
                lineGraphs.redrawFlag=true
                lineGraphs.requestPaint()
            }
            onWorkoutGoalTimeChanged: {
                targetGraphs.redrawFlag=true
                targetGraphs.requestPaint()
                lineGraphs.redrawFlag=true
                lineGraphs.requestPaint()
            }
        }

        Connections {
            target: root
            onVisibleChanged: {
                if (root.visible) {
                    lineGraphs.redrawFlag=true
                    lineGraphs.redrawGraphs()
                }
            }
        }

        Timer {
            interval: lineGraphsView.timerInterval
            repeat: true
            running: aWorkoutRunData.totalTime > 0
            onTriggered: {                
                if(aWorkoutRunData.totalTime > (lineGraphsView.scrollLockTime + lineGraphsView.startT)) {
                    lineGraphsView.scrollFlag = true
                    lineGraphsView.startT += lineGraphsView.scrollTime
//                    console.log("Scroll Line Graphs " + aWorkoutRunData.totalTime + ">" + lineGraphsView.scrollLockTime + "+" + (lineGraphsView.startT-lineGraphsView.scrollTime)  + ", startT: "+ lineGraphsView.startT + ", startX: " + lineGraphsView.startX)
                    targetGraphs.requestPaint()
                } else
                    lineGraphsView.scrollFlag = false

                lineGraphs.requestPaint()
            }
        }
    }

    Component.onCompleted: {
        lineGraphsView.isHRC = workoutProgram.isHRCProgram
//        gridGraph.requestPaint()
        if(lineGraphsView.isTarget1 || lineGraphsView.isTarget2 || workoutProgram.isHRCProgram) {
            targetGraphs.requestPaint()
        }

    }

    TextLabel {
        id: hr_label
        visible: heartRateManager.heartRateAvailable
        color: "#EC1A24"
        text: qsTr("HR") + application.emptyString
        horizontalAlignment: Text.AlignRight

        anchors {
            right: lineGraphsView.left
            rightMargin: 5 * main.hScaleFactor
            bottom: lineGraphsView.bottom
            bottomMargin: 0.65*lineGraphsView.height
        }
    }

    TextLabel {
        id: pace_label
        color: Qt.rgba(60/255,180/255,244/255, 1.0)
        text: ((root.isTreadmill || main.isStepmill) ? qsTr("SPEED") : qsTr("PACE")) + application.emptyString
        horizontalAlignment: Text.AlignRight

        anchors {
            right: lineGraphsView.left
            rightMargin: 5 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 0.35*lineGraphsView.height
        }
    }

    TextLabel {
        id: watts_label
        visible: !main.isStepmill
        color: "#3FDD00"
        text: (root.isTreadmill ? qsTr("INCLINE") : qsTr("WATTS")) + application.emptyString
        font.letterSpacing: 0
        horizontalAlignment: Text.AlignRight

        anchors {
            right: lineGraphsView.left
            rightMargin: 5 * main.hScaleFactor
            bottom: lineGraphsView.bottom
            bottomMargin: 0.2*lineGraphsView.height
        }
    }
}
