import QtQuick 2.12
import ".."
import WorkoutManager 1.0
import WorkoutRunData 1.0
import Components 1.0

Item {
    id: root

    property int startElevation: workoutProgram.startElevation
    property int endElevation: workoutProgram.endElevation
    property int currentUserElevation: startElevation + aWorkoutRunData.workoutFeetClimbed
    property string climbTimeRemaining: Globals.secondsToMSString(aWorkoutRunData.climbTimeRemaining)
    property string projectedFinishTime: getTimeRemaining()
    property string greenFlagIcon: "../graphics/ui3_0/Monuments/Alpine_GreenFlag.png"
    property string checkeredFlagIcon: "../graphics/ui3_0/Monuments/Alpine_CheckeredFlag.png"
    property point flagLocation: Qt.point(-1, -1)
    property string backgroundImage: ""
    property int imageMarginX: main.screenWidth * 1604 / 1920 - viewSizeX
    property int imageMarginY: height - viewSizeY
    property int viewSizeX: main.screenWidth * 1589 / 1920
    property int viewSizeY: main.screenHeight * 729 / 1080
    property string fontName: main.scaleFactor > 1 ? "bold 24px \"FreeSans\"" : "bold 17px \"FreeSans\""

    width: main.screenWidth * 1604 / 1920
    height: main.screenHeight * 743 / 1080
    anchors.verticalCenter: parent.verticalCenter
    anchors.left: parent.left

    property bool isMenuDisplayed: false
    property bool isMenuMoving: false
    signal action()

    Connections {
        target: currentViewName === "MONUMENT_VIEW" && visible && opacity > 0 ? keymapper : null

        onEnter2Pressed: {
            main.debugKeys("enter2Pressed", "MonumentViewContainer")
        }
    }

    onEndElevationChanged: precomputeFlagLocation()

    function getPreloadImages() {
        switch (workoutProgram.programIndex) {
            case WorkoutProgram.MOUNT_EVEREST_PROG:
            case WorkoutProgram.STEP_MOUNT_EVEREST_PROG:
                return ["../graphics/ui3_0/Monuments/ALPINE_everest.png"]
            case WorkoutProgram.EIFFEL_TOWER_PROG:
            case WorkoutProgram.STEP_EIFFEL_TOWER_PROG:
                return ["../graphics/ui3_0/Monuments/ALPINE_eiffel_begining.png", "../graphics/ui3_0/Monuments/ALPINE_eiffel_ending.png"]
            case WorkoutProgram.GATEWAY_ARCH_PROG:
            case WorkoutProgram.STEP_GATEWAY_ARCH_PROG:
                return ["../graphics/ui3_0/Monuments/ALPINE_arch_begining.png", "../graphics/ui3_0/Monuments/ALPINE_arch_ending.png"]
            case WorkoutProgram.MOUNT_KILIMANJARO_PROG:
            case WorkoutProgram.STEP_MOUNT_KILIMANJARO_PROG:
                return ["../graphics/ui3_0/Monuments/ALPINE_kilimanjaro.png"]
            case WorkoutProgram.MOUNT_RAINIER_PROG:
            case WorkoutProgram.STEP_MOUNT_RAINIER_PROG:
                return ["../graphics/ui3_0/Monuments/ALPINE_rainier.png"]
        }
        return []
    }

    function getRenderPath() {
        switch (workoutProgram.programIndex) {
            case WorkoutProgram.MOUNT_EVEREST_PROG:
            case WorkoutProgram.STEP_MOUNT_EVEREST_PROG:
                return mtEverestRenderPath
            case WorkoutProgram.MOUNT_KILIMANJARO_PROG:
            case WorkoutProgram.STEP_MOUNT_KILIMANJARO_PROG:
                return kilimanjaroRenderPath
            case WorkoutProgram.MOUNT_RAINIER_PROG:
            case WorkoutProgram.STEP_MOUNT_RAINIER_PROG:
                return mtRainierRenderPath
        }
        return null
    }

    function getTextItems() {
        switch (workoutProgram.programIndex) {
            case WorkoutProgram.MOUNT_EVEREST_PROG:
            case WorkoutProgram.STEP_MOUNT_EVEREST_PROG:
                return mtEverestTextItems
            case WorkoutProgram.MOUNT_KILIMANJARO_PROG:
            case WorkoutProgram.STEP_MOUNT_KILIMANJARO_PROG:
                return kilimanjaroTextItems
            case WorkoutProgram.MOUNT_RAINIER_PROG:
            case WorkoutProgram.STEP_MOUNT_RAINIER_PROG:
                return mtRainierTextItems
        }
        return null
    }

    ListModel {
        id: mtEverestRenderPath
        // NOTE: The elevation for points inbetween base camps is interpolated
        // based on the y-value (this is done manually in the list below)
        ListElement {x: 84; y: 473; elevation: 0 }
        ListElement {x: 208; y: 434; elevation: 5667 }
        ListElement {x: 294; y: 421; elevation: 7556 }
        ListElement {x: 463; y: 358; elevation: 17000 } //BASE CAMP
        ListElement {x: 500; y: 320; elevation: 18024 }
        ListElement {x: 547; y: 304; elevation: 18433 }
        ListElement {x: 586; y: 280; elevation: 19047 }
        ListElement {x: 632; y: 258; elevation: 19900 } //Lho La
        ListElement {x: 717; y: 217; elevation: 21300 } //Camp 2
        ListElement {x: 739; y: 165; elevation: 24500 } //Geneva Spur
        ListElement {x: 694; y: 89; elevation: 26000 } //Camp 4
        ListElement {x: 592; y: 32; elevation: 29029 } //SUMMIT
    }

    ListModel {
        id: kilimanjaroRenderPath
        ListElement {x: 208; y: 473; elevation: 0 }
        ListElement {x: 309; y: 438; elevation: 1139 }
        ListElement {x: 393; y: 411; elevation: 2025 }
        ListElement {x: 446; y: 365; elevation: 3543 }
        ListElement {x: 519; y: 333; elevation: 4597 }
        ListElement {x: 565; y: 294; elevation: 5905 } //Machame Gate
        ListElement {x: 677; y: 268; elevation: 7574 }
        ListElement {x: 740; y: 219; elevation: 10746 }
        ListElement {x: 720; y: 185; elevation: 13000 } //Shira Peak
        ListElement {x: 707; y: 164; elevation: 13773 }
        ListElement {x: 623; y: 127; elevation: 15174 } //Lava Tower
        ListElement {x: 591; y: 66; elevation: 16893 } //Mawenzi Peak
        ListElement {x: 500; y: 18; elevation: 19341 } //Uhuru Peak
    }

    ListModel {
        id: mtRainierRenderPath
        ListElement {x: 317; y: 473; elevation: 0 }
        ListElement {x: 410; y: 429; elevation: 1380 }
        ListElement {x: 436; y: 389; elevation: 2639 }
        ListElement {x: 528; y: 362; elevation: 3491 }
        ListElement {x: 565; y: 330; elevation: 4547 }
        ListElement {x: 638; y: 302; elevation: 5440 } //Alta Vista
        ListElement {x: 682; y: 260; elevation: 6785 }
        ListElement {x: 795; y: 201; elevation: 8717 }
        ListElement {x: 818; y: 156; elevation: 10188 } //Camp Muir
        ListElement {x: 764; y: 129; elevation: 10659 }
        ListElement {x: 718; y: 73; elevation: 11600 } //Disappointment Cleaver
        ListElement {x: 620; y: 47; elevation: 12796 }
        ListElement {x: 562; y: 18; elevation: 14411 } //Colombia Crest
    }

    ListModel {
        id: mtEverestTextItems
        ListElement {x: 506; y: 347; label: "BASE CAMP"; elevFeet: 17000; elevMeters: 5182 }
        ListElement {x: 669; y: 250; label: "Lho La"; elevFeet: 19900; elevMeters: 6066 }
        ListElement {x: 752; y: 208; label: "Camp 2"; elevFeet: 21300; elevMeters: 6492 }
        ListElement {x: 772; y: 151; label: "Geneva Spur"; elevFeet: 24500; elevMeters: 7468 }
        ListElement {x: 727; y: 75; label: "Camp 4"; elevFeet: 26000; elevMeters: 7925 }
        ListElement {x: 619; y: 17; label: "SUMMIT"; elevFeet: 29029; elevMeters: 8848 }
    }

    ListModel {
        id: mtRainierTextItems
        ListElement {x: 666; y: 294; label: "Alta Vista"; elevFeet: 5440; elevMeters: 1658 }
        ListElement {x: 844; y: 136; label: "Camp Muir"; elevFeet: 10188; elevMeters: 3105 }
        ListElement {x: 751; y: 57; label: "Disappointment Cleaver"; elevFeet: 11600; elevMeters: 3536 }
        ListElement {x: 609; y: 1; label: "Colombia Crest Summit"; elevFeet: 14411; elevMeters: 4392 }
    }

    ListModel {
        id: kilimanjaroTextItems
        ListElement {x: 580; y: 296; label: "Machame Gate"; elevFeet: 5905; elevMeters: 1800 }
        ListElement {x: 751; y: 168; label: "Shira Peak"; elevFeet: 13000; elevMeters: 3962 }
        ListElement {x: 655; y: 107; label: "Lava Tower Camp"; elevFeet: 15174; elevMeters: 4625 }
        ListElement {x: 621; y: 55; label: "Mawenzi Peak"; elevFeet: 16893; elevMeters: 5149 }
        ListElement {x: 545; y: 3; label: "Uhuru Peak"; elevFeet: 19341; elevMeters: 5895 }
    }

    ListModel {
        id: eiffelTowerPlatforms
        ListElement { height: 187; label: QT_TR_NOOP("FIRST PLATFORM") }
        ListElement { height: 377; label: QT_TR_NOOP("SECOND PLATFORM") }
        ListElement { height: 906; label: QT_TR_NOOP("THIRD PLATFORM") }
        ListElement { height: 1063; label: QT_TR_NOOP("OFFICIAL HEIGHT") }
    }

    Timer {
        interval: 1000 / 60.0
        repeat: true
        running: currentViewName === "MONUMENT_VIEW"
        onTriggered: {
            canvas.requestPaint()
        }
    }

    function getTimeRemaining() {
        return (main.isStepmill || (main.isTreadmill && workoutManager.treadIncline > 0)) && workoutManager.treadSpeed > 0
                ? Globals.secondsToMSString(aWorkoutRunData.projectedFinishTime) : "--:--"
    }

    function paintTimeRemaining() {
        if (aWorkoutRunData.mode !== WorkoutRunData.WORKOUT_MODE) {
            return
        }

        var ctx = canvas.getContext("2d")
        ctx.fillStyle = Qt.rgba(0.0, 0.0, 0.0, 1.0)
        ctx.font = root.fontName
        ctx.textBaseline = "top"
        var timeRemainingText = qsTr("PROJECTED FINISH TIME") + application.emptyString
        ctx.fillText(timeRemainingText, 15, 10)
        ctx.font = 'bold 48px "FreeSans"'
        ctx.fillText(root.projectedFinishTime, 15, 25)

    }

    function paintEiffelTower() {

        var progressLineWidth = 0.3*root.viewSizeX
        var platformLineWidth = 0.1*root.viewSizeX
        var edgeMargin = 10*main.scaleFactor
        var textMargin = 25*main.scaleFactor

        var ctx = canvas.getContext("2d")
        ctx.drawImage("../graphics/ui3_0/Monuments/ALPINE_eiffel_ending.png", 0, 0, root.viewSizeX, root.viewSizeY)
        ctx.save()

        var baselineY = 0.05*root.viewSizeY
        var baseLineX = 0.645*root.viewSizeX
        var percentComplete = root.currentUserElevation / workoutProgram.endElevation
        var trackHeight = root.viewSizeY - baselineY - edgeMargin
        var completeY = baselineY + parseInt(trackHeight * percentComplete)
        // Bound to top of screen
        completeY = Math.min(completeY, baselineY + trackHeight)
        // And to bottom of screen
        completeY = Math.max(edgeMargin, completeY)

        // Make coordinates relative to top
        completeY = root.viewSizeY - completeY
        baselineY = root.viewSizeY - baselineY

        ctx.save()
        ctx.beginPath()
        ctx.moveTo(0,0)
        ctx.lineTo(0, completeY)
        ctx.lineTo(root.viewSizeX, completeY)
        ctx.lineTo(root.viewSizeX, 0)
        ctx.closePath()
        ctx.clip()
        ctx.drawImage("../graphics/ui3_0/Monuments/ALPINE_eiffel_begining.png", 0, 0, root.viewSizeX, root.viewSizeY)

        ctx.restore() // Forget about clipping region

        paintTimeRemaining()

        ctx.strokeStyle = Qt.rgba(0.980, 0.650, 0.102, 1.0) // Yellow
        ctx.lineWidth = 5*main.scaleFactor
        // Draw base line
        ctx.beginPath()
        ctx.moveTo(0.05*root.viewSizeX, baselineY)
        ctx.lineTo(0.95*root.viewSizeX, baselineY)
        ctx.stroke()

        // Paint vertical rising line
        ctx.beginPath()
        ctx.moveTo(baseLineX, baselineY)
        ctx.lineTo(baseLineX, edgeMargin)
        ctx.stroke()

        // Paint ladder bars for platform readings
        for (var i = 0; i < eiffelTowerPlatforms.count; i++) {
            var item = eiffelTowerPlatforms.get(i)
            ctx.beginPath()
            var lineY = parseInt((item.height / eiffelTowerPlatforms.get(eiffelTowerPlatforms.count - 1).height) * (baselineY - edgeMargin))
            lineY = baselineY - lineY
            ctx.moveTo(baseLineX, lineY)
            ctx.lineTo(baseLineX - platformLineWidth, lineY)
            ctx.stroke()

            ctx.beginPath()
            ctx.fillStyle = Qt.rgba(0.0, 0.0, 0.0, 1.0)
            ctx.textBaseline = "middle"
            ctx.font = root.fontName
            var elevation = Globals.formatWithSeparator(workoutScreen.unitState === "METRIC" ? Globals.feetToMeters(item.height).toFixed(0) : item.height)
            elevation = elevation + " " + (workoutScreen.unitState === "METRIC" ? qsTr("Meters") : qsTr("FT")) + application.emptyString
            var str = elevation + " " + qsTr(item.label)
            var strWidth = ctx.measureText(str).width
            ctx.fillText(str, baseLineX+textMargin, lineY + edgeMargin)
            // Draw flag if platform has been reached
            if (root.currentUserElevation >= item.height) {
                ctx.drawImage(root.greenFlagIcon, baseLineX + strWidth + textMargin + edgeMargin, lineY - edgeMargin)
            }
        }

        // Display current elevation
        ctx.textBaseline = "top"
        ctx.font = root.fontName
        var userElev = Globals.formatWithSeparator(workoutScreen.unitState === "METRIC" ? Globals.feetToMeters(root.currentUserElevation).toFixed(0) : root.currentUserElevation)
        userElev = userElev + " " + (workoutScreen.unitState === "METRIC" ? qsTr("Meters") : qsTr("FT")) + application.emptyString
        var measured = ctx.measureText(userElev)
        ctx.fillText(userElev, baseLineX - progressLineWidth - measured.width - textMargin - edgeMargin, completeY - edgeMargin)

        // Paint red progress bar
        ctx.strokeStyle = Qt.rgba(0.925, 0.110, 0.141, 1.0)
        ctx.fillStyle = Qt.rgba(0.925, 0.110, 0.141, 1.0)
        ctx.lineWidth = 10*main.scaleFactor
        ctx.lineCap = "round"
        ctx.beginPath()
        ctx.moveTo(baseLineX - progressLineWidth - textMargin, completeY)
        ctx.lineTo(baseLineX - textMargin, completeY)
        ctx.stroke()

    }

    function paintGatewayArch() {

        var progressLineWidth = 0.4*root.viewSizeX
        var platformLineWidth = 0.1*root.viewSizeX
        var edgeMargin = 10*main.scaleFactor
        var textMargin = 25*main.scaleFactor

        var ctx = canvas.getContext("2d")
        ctx.drawImage("../graphics/ui3_0/Monuments/ALPINE_arch_ending.png", 0, 0, root.viewSizeX, root.viewSizeY)
        ctx.save()

        var baselineY = 0.07*root.viewSizeY
        var baseLineX = 0.676*root.viewSizeX
        var percentComplete = root.currentUserElevation / workoutProgram.endElevation
        var trackHeight = root.viewSizeY - baselineY - edgeMargin
        var completeY = baselineY + parseInt(trackHeight * percentComplete)
        // Bound to top of screen
        completeY = Math.min(completeY, baselineY + trackHeight)
        // And to bottom of screen
        completeY = Math.max(edgeMargin, completeY)

        // Make coordinates relative to top
        completeY = root.viewSizeY - completeY
        baselineY = root.viewSizeY - baselineY

        ctx.save()
        ctx.beginPath()
        ctx.moveTo(0,0)
        ctx.lineTo(0, completeY)
        ctx.lineTo(root.viewSizeX, completeY)
        ctx.lineTo(root.viewSizeX, 0)
        ctx.closePath()
        ctx.clip()
        ctx.drawImage("../graphics/ui3_0/Monuments/ALPINE_arch_begining.png", 0, 0, root.viewSizeX, root.viewSizeY)

        ctx.restore() // Forget about clipping region

        paintTimeRemaining()

        ctx.strokeStyle = Qt.rgba(0.980, 0.650, 0.102, 1.0) // Yellow
        ctx.lineWidth = 5*main.scaleFactor
        // Draw base line
        ctx.beginPath()
        ctx.moveTo(0.05*root.viewSizeX, baselineY)
        ctx.lineTo(0.95*root.viewSizeX, baselineY)
        ctx.stroke()

        // Paint vertical rising line
        ctx.beginPath()
        ctx.moveTo(baseLineX, baselineY)
        ctx.lineTo(baseLineX, edgeMargin)
        ctx.stroke()

        // Paint ladder bar - Arch only has one
        ctx.beginPath()
        ctx.moveTo(baseLineX, edgeMargin)
        ctx.lineTo(baseLineX-platformLineWidth, edgeMargin)
        ctx.stroke()
        ctx.beginPath()
        ctx.fillStyle = Qt.rgba(0.0, 0.0, 0.0, 1.0)
        ctx.textBaseline = "top"
        ctx.font = root.fontName
        var ladderText = qsTr("OFFICIAL HEIGHT") + " " + (workoutScreen.unitState === "METRIC" ? "192 "+qsTr("Meters") : "630 "+qsTr("FT")) + application.emptyString
        ctx.fillText(ladderText, baseLineX+textMargin, 3)

        // Display current elevation
        ctx.beginPath()
        ctx.fillStyle = Qt.rgba(1.0, 1.0, 1.0, 1.0)
        ctx.textBaseline = "top"
        ctx.font = root.fontName
        var userElev = Globals.formatWithSeparator(workoutScreen.unitState === "METRIC" ? Globals.feetToMeters(root.currentUserElevation).toFixed(0) : root.currentUserElevation)
        userElev = userElev + " " + (workoutScreen.unitState === "METRIC" ? qsTr("Meters") : qsTr("FT")) + application.emptyString
        var measured = ctx.measureText(userElev)
        ctx.fillText(userElev, baseLineX-progressLineWidth - measured.width - textMargin - edgeMargin, completeY - edgeMargin)

        // Paint red progress bar
        ctx.strokeStyle = Qt.rgba(0.925, 0.110, 0.141, 1.0)
        ctx.fillStyle = Qt.rgba(0.925, 0.110, 0.141, 1.0)
        ctx.lineWidth = 10*main.scaleFactor
        ctx.lineCap = "round"
        ctx.beginPath()
        ctx.moveTo(baseLineX-progressLineWidth-textMargin, completeY)
        ctx.lineTo(baseLineX-textMargin, completeY)
        ctx.stroke()

    }

    function closeEnoughForFlag(height1, height2) {
        return Math.abs(height1 - height2) <= 100
    }

    function precomputeFlagLocation() {
        // The flag location doesn't change, so precompute it to be faster
        if (workoutProgram.programIndex !== WorkoutProgram.MOUNT_EVEREST_PROG &&
            workoutProgram.programIndex !== WorkoutProgram.MOUNT_KILIMANJARO_PROG &&
            workoutProgram.programIndex !== WorkoutProgram.MOUNT_RAINIER_PROG &&
            workoutProgram.programIndex !== WorkoutProgram.STEP_MOUNT_EVEREST_PROG &&
            workoutProgram.programIndex !== WorkoutProgram.STEP_MOUNT_KILIMANJARO_PROG &&
            workoutProgram.programIndex !== WorkoutProgram.STEP_MOUNT_RAINIER_PROG) {
            // Only applies to mountain workouts
            return
        }

        var renderPath = getRenderPath()
        var textItems = getTextItems()
        for (var i = 0; i < textItems.count; i++) {
            var item = textItems.get(i)
            // If target is close to a base camp, wait and put the flag after the text, drawn later
            if (closeEnoughForFlag(root.endElevation, item.elevFeet)) {
                return
            }
        }

        for (var i = 0; i < renderPath.count - 1; i++) {
            if (root.endElevation >= renderPath.get(i).elevation && root.endElevation <= renderPath.get(i+1).elevation) {
                var progress = (root.endElevation - renderPath.get(i).elevation) / (renderPath.get(i+1).elevation - renderPath.get(i).elevation)
                var slope = (renderPath.get(i+1).y - renderPath.get(i).y) / (renderPath.get(i+1).x - renderPath.get(i).x)
                var dx = (progress * (renderPath.get(i+1).x - renderPath.get(i).x))
                var x = renderPath.get(i).x + dx
                var dy = slope * dx
                var y = renderPath.get(i).y + dy
                root.flagLocation = Qt.point(Math.round(x*main.hScaleFactor), Math.round(y*main.vScaleFactor))
            }
        }

    }

    function paintMountain() {
        var edgeMargin = 10*main.scaleFactor
        var textMargin = 25*main.scaleFactor
        var dotSize = 26*main.scaleFactor
        var ctx = canvas.getContext("2d")

        var renderPath = getRenderPath()

        ctx.drawImage(root.backgroundImage, 0, 0, root.viewSizeX, root.viewSizeY)
        ctx.save()

        paintTimeRemaining()

        ctx.strokeStyle = Qt.rgba(0.980, 0.650, 0.102, 1.0)
        ctx.lineWidth = 5*main.scaleFactor
        ctx.beginPath()
        var startPoint = renderPath.get(0)
        ctx.moveTo(Math.round(startPoint.x*main.hScaleFactor), Math.round(startPoint.y*main.vScaleFactor))
        for (var i = 1; i < renderPath.count; i++) {
            ctx.lineTo(Math.round(renderPath.get(i).x*main.hScaleFactor), Math.round(renderPath.get(i).y*main.vScaleFactor))
        }
        ctx.stroke()

        // Draw base line
        var baselineY = 0.91*root.viewSizeY
        ctx.beginPath()
        ctx.moveTo(0.05*root.viewSizeX, baselineY)
        ctx.lineTo(0.95*root.viewSizeX, baselineY)
        ctx.stroke()

        // Track points for drawing red line (user progress)
        var indexBegin = 0
        var indexEnd = 0
        var pointBegin = null
        var pointEnd = null
        for (var i = 0; i < renderPath.count - 1; i++) {
            if (pointBegin && pointEnd) {
                break
            }
            if (root.startElevation >= renderPath.get(i).elevation && root.startElevation <= renderPath.get(i+1).elevation) {
                // Percent progress along this line
                var progress = (root.startElevation - renderPath.get(i).elevation) / (renderPath.get(i+1).elevation - renderPath.get(i).elevation)
                var slope = (renderPath.get(i+1).y - renderPath.get(i).y) / (renderPath.get(i+1).x - renderPath.get(i).x)
                var dx = (progress * (renderPath.get(i+1).x - renderPath.get(i).x))
                var x = renderPath.get(i).x + dx
                var dy = slope * dx
                var y = renderPath.get(i).y + dy
                pointBegin = [Math.round(x*main.hScaleFactor), Math.round(y*main.vScaleFactor)]
                indexBegin = i
            }
            if (root.currentUserElevation >= renderPath.get(i).elevation && root.currentUserElevation <= renderPath.get(i+1).elevation) {
                // Percent progress along this line
                var progress = (root.currentUserElevation - renderPath.get(i).elevation) / (renderPath.get(i+1).elevation - renderPath.get(i).elevation)
                var slope = (renderPath.get(i+1).y - renderPath.get(i).y) / (renderPath.get(i+1).x - renderPath.get(i).x)
                var dx = (progress * (renderPath.get(i+1).x - renderPath.get(i).x))
                var x = renderPath.get(i).x + dx
                var dy = slope * dx
                var y = renderPath.get(i).y + dy
                pointEnd = [Math.round(x*main.hScaleFactor), Math.round(y*main.vScaleFactor)]
                indexEnd = i
            }
        }
        if (!pointEnd) {
            if (root.currentUserElevation < 0) {
                // They descended below 0ft
                var pt = renderPath.get(0)
                pointEnd = [Math.round(pt.x*main.hScaleFactor), Math.round(pt.y*main.vScaleFactor)]
                indexEnd = 0
            } else {
                // They went past the endpoint
                var pt = renderPath.get(renderPath.count - 1)
                pointEnd = [Math.round(pt.x*main.hScaleFactor), Math.round(pt.y*main.vScaleFactor)]
                indexEnd = renderPath.count - 1
            }
        }

        // Draw checkered flag (unless it is drawn after text, further down)
        if (root.flagLocation.x > -1) {
            // 32x31 is the checkered flag image width/height
            ctx.drawImage(root.checkeredFlagIcon, root.flagLocation.x, root.flagLocation.y - 32)
        }

        //console.log("Points", pointBegin, pointEnd)

        ctx.strokeStyle = Qt.rgba(0.925, 0.110, 0.141, 1.0)
        ctx.fillStyle = Qt.rgba(0.925, 0.110, 0.141, 1.0)
        ctx.lineWidth = 10*main.scaleFactor
        ctx.lineCap = "round"
        ctx.beginPath()
        if (root.currentUserElevation < root.startElevation) {
            // User descended backwards from start point
            ctx.moveTo(pointEnd[0], pointEnd[1])
            for (var i = indexEnd + 1; i <= indexBegin; i++) {
                var pt = renderPath.get(i)
                ctx.lineTo(Math.round(pt.x*main.hScaleFactor), Math.round(pt.y*main.vScaleFactor))
            }
            ctx.lineTo(pointBegin[0], pointBegin[1])
        } else {
            ctx.moveTo(pointBegin[0], pointBegin[1])
            for (var i = indexBegin + 1; i <= indexEnd; i++) {
                var pt = renderPath.get(i)
                ctx.lineTo(Math.round(pt.x*main.hScaleFactor), Math.round(pt.y*main.vScaleFactor))
            }
            ctx.lineTo(pointEnd[0], pointEnd[1])
        }

        ctx.stroke()

        ctx.beginPath()
        ctx.ellipse(pointEnd[0] - dotSize/2, pointEnd[1] - dotSize/2, dotSize, dotSize)
        ctx.fill()

        ctx.beginPath()
        ctx.fillStyle = Qt.rgba(0.0, 0.0, 0.0, 1.0)
        ctx.textBaseline = "top"
        ctx.font = root.fontName
        var textItems = getTextItems()
        for (var i = 0; i < textItems.count; i++) {
            var str
            var item = textItems.get(i)
            var elev = Globals.formatWithSeparator(workoutScreen.unitState === "METRIC" ? item.elevMeters : item.elevFeet)
            elev = elev + " " + (workoutScreen.unitState === "METRIC" ? qsTr("Meters") : qsTr("FT")) + application.emptyString
            str = elev + " " + qsTr(item.label)
            ctx.fillText(str, item.x*main.hScaleFactor, item.y*main.vScaleFactor)
            var strWidth = ctx.measureText(str).width
            if (closeEnoughForFlag(root.endElevation, item.elevFeet)) {
                // Draw checkered flag if target is close enough to this point
                var flagY = item.y*main.vScaleFactor - edgeMargin
                if (flagY < 2) {
                    // Avoid painting over top of screen
                    flagY = 2
                }

                ctx.drawImage(root.checkeredFlagIcon, item.x*main.hScaleFactor + strWidth + 5, flagY)
            } else {
                // Otherwise, draw green flag if they reached this point
                if (root.currentUserElevation >= item.elevFeet && item.elevFeet > root.startElevation) {
                    if (i !== textItems.count - 1) {
                        ctx.drawImage(root.greenFlagIcon, item.x*main.hScaleFactor + strWidth + edgeMargin, item.y*main.vScaleFactor)
                    }
                }
            }
        }

        // Display user elevation
        ctx.textBaseline = "top"
        var elevation = Globals.formatWithSeparator(workoutScreen.unitState === "METRIC" ? Globals.feetToMeters(root.currentUserElevation).toFixed(0) : root.currentUserElevation)
        elevation = elevation + " " + (workoutScreen.unitState === "METRIC" ? qsTr("Meters") : qsTr("FT")) + application.emptyString
        var measured = ctx.measureText(elevation)
        ctx.fillText(elevation, pointEnd[0] - measured.width - textMargin - edgeMargin, pointEnd[1] - edgeMargin)

        ctx.restore()
    }

    Canvas {
        id: canvas
        anchors.fill: parent
        anchors.leftMargin: root.imageMarginX/2
        anchors.rightMargin: root.imageMarginX/2
        anchors.topMargin: root.imageMarginY/2
        anchors.bottomMargin: root.imageMarginY/2
        onPaint: {
            switch (workoutProgram.programIndex) {
                case WorkoutProgram.MOUNT_EVEREST_PROG:
                case WorkoutProgram.MOUNT_KILIMANJARO_PROG:
                case WorkoutProgram.MOUNT_RAINIER_PROG:
                case WorkoutProgram.STEP_MOUNT_EVEREST_PROG:
                case WorkoutProgram.STEP_MOUNT_KILIMANJARO_PROG:
                case WorkoutProgram.STEP_MOUNT_RAINIER_PROG:
                    paintMountain()
                    return
                case WorkoutProgram.EIFFEL_TOWER_PROG:
                case WorkoutProgram.STEP_EIFFEL_TOWER_PROG:
                    paintEiffelTower()
                    return
                case WorkoutProgram.GATEWAY_ARCH_PROG:
                case WorkoutProgram.STEP_GATEWAY_ARCH_PROG:
                    paintGatewayArch()
                    break
            }

        }

        Component.onCompleted: {
            var images = getPreloadImages()
            for (var i = 0; i < images.length; i++) {
                canvas.loadImage(images[i])
            }
            if (images.length === 1) {
                // Store image for mountain
                root.backgroundImage = images[0]
            }

            precomputeFlagLocation()
        }
    }

}
