import QtQuick 2.12
//import TrackView 1.0
import ".."
import Components 1.0


Item{
    id: root

    property bool isMenuDisplayed: false
    property bool isMenuMoving: false
    signal action()

    Timer {
        id: timer
        interval: 1000 / 60.0
        repeat: true
        onTriggered: {
            canvas.requestPaint()
        }
    }

    function shown() {
        timer.start()
        // No TV or Browser audio when viewing the track, only Bluetooth.
        audioManager.btRequest()
    }

    function hidden() {
        timer.stop()
    }

    function paintTrack()
    {
        if(!canvas.available)
            return

        var ctx = canvas.getContext("2d")

        var viewSizeX = 0.69 * root.width
        var viewSizeY = 0.86 * root.height
        var offsetX = (root.width - viewSizeX)/2
        var offsetY = (root.height - viewSizeY)/2
        var trackLineWidth = 5 * main.scaleFactor
        var dotWidth = 30 * main.scaleFactor

        var trackArcRadius = 0.45*viewSizeY;
        var trackStraightL2 = 0.25*viewSizeX;

        var centerX = viewSizeX/2 + offsetX;
        var centerY = viewSizeY/2 + offsetY;

        var rectangleRightX = centerX+trackStraightL2;
        var rectangleLeftX = centerX-trackStraightL2;

        var rectangleUpperY = centerY-trackArcRadius;
        var rectangleLowerY = centerY+trackArcRadius;

        var startAngle = Math.PI/2;

        // reference background rectangle
        //ctx.fillStyle = Qt.rgba(0.7, 0.7, 0.7, 1)
        ctx.clearRect(0, 0, root.width, root.height)

        // Paint track
        ctx.strokeStyle = Qt.rgba(128.0/255, 128.0/255, 128.0/255, 1.0) // grey
        ctx.lineWidth = trackLineWidth

        ctx.beginPath()
        ctx.moveTo(rectangleLeftX, rectangleLowerY)
        ctx.arc(rectangleRightX, centerY, trackArcRadius, startAngle, -startAngle, true)
        ctx.arc(rectangleLeftX, centerY, trackArcRadius, -startAngle,startAngle, true)
        ctx.stroke()

        // Paint progress along track
        ctx.strokeStyle = Qt.rgba(60/255,180/255,244/255, 1.0) // #3CB4F4

        var trackDistance = 250.0
        var segmentDistance = trackDistance/4
        //var laps = 0

        var trackPosition = aWorkoutRunData.trackDistance
        var laps = Math.floor(trackPosition/trackDistance)
        var segment = trackPosition-laps*trackDistance
        var trackIconX = centerX
        var trackIconY = centerY

        ctx.beginPath()
        if(segment <= segmentDistance)
        {
            var fillAngle = Math.PI * segment / segmentDistance
            ctx.arc(rectangleRightX, centerY, trackArcRadius, startAngle, startAngle-fillAngle, true)

            trackIconX = rectangleRightX + (trackArcRadius * Math.sin(fillAngle))
            trackIconY = centerY + (trackArcRadius * Math.cos(fillAngle))
        }
        else if(segment <= 2*segmentDistance)
        {
            ctx.arc(rectangleRightX, centerY, trackArcRadius, startAngle, -startAngle, true)

            var percent = (segment - trackDistance/4) / segmentDistance
            var linearDist =  rectangleRightX - 2*trackStraightL2*percent

            ctx.lineTo(linearDist, rectangleUpperY)

            trackIconX = linearDist
            trackIconY = rectangleUpperY
        }
        else if(segment <= 3*segmentDistance)
        {
            ctx.arc(rectangleRightX, centerY, trackArcRadius, startAngle, -startAngle, true)

            ctx.lineTo(rectangleLeftX, rectangleUpperY)

            var fillAngle = Math.PI * (segment - 2*segmentDistance) / segmentDistance
            ctx.arc(rectangleLeftX, centerY, trackArcRadius, -startAngle, -startAngle-fillAngle, true)

            trackIconX = rectangleLeftX - (trackArcRadius * Math.sin(fillAngle))
            trackIconY = centerY - (trackArcRadius * Math.cos(fillAngle))
        }
        else
        {
            ctx.arc(rectangleRightX, centerY, trackArcRadius, startAngle, -startAngle, true)
            ctx.lineTo(rectangleLeftX, rectangleUpperY)

            ctx.arc(rectangleLeftX, centerY, trackArcRadius, -startAngle, startAngle, true)

            var percent = (segment - 3*segmentDistance) / segmentDistance
            var linearDist =  rectangleLeftX + 2*trackStraightL2*percent

            ctx.lineTo(linearDist, rectangleLowerY)

            trackIconX = linearDist
            trackIconY = rectangleLowerY
        }
        ctx.stroke()

        // Paint Current Position Dot
        ctx.beginPath()
        ctx.ellipse(trackIconX-dotWidth/2, trackIconY-dotWidth/2, dotWidth, dotWidth)
        ctx.fillStyle = Qt.rgba(1.0, 1.0, 1.0, 1.0)
        ctx.fill()

        // Display Lap text
        var lapText = qsTr("LAP") + application.emptyString
        var lapY = rectangleLowerY - 0.03*viewSizeY
        var lapX = rectangleRightX-0.03*viewSizeX
        var measured = ctx.measureText(lapText)
        var laptextN = (laps+1) + application.emptyString

        ctx.fillStyle = Qt.rgba(1.0, 1.0, 1.0, 1.0)
        ctx.textBaseline = "bottom"
        ctx.font = 'bold 16px "Montserrat Hairline"'

        //ctx.beginPath()
        ctx.fillText(lapText, lapX, lapY)

        //ctx.beginPath()
        ctx.font = '66px "FreeSans"'
        ctx.fillText(laptextN, lapX+measured.width/3, lapY+5*main.vScaleFactor)

    }

    Canvas {
        id: canvas
        anchors.fill: parent
        onPaint: {
            paintTrack()
        }

        Component.onCompleted: {
            paintTrack()
        }
    }
}
