import QtQuick 2.12
import "../string_format.js" as StrFormat
import Components 1.0

Item {
    id: root

    property bool isTreadmill: true
    property int incline:105
    property int speed: 0
    property string speedUnits
//    property bool showHR: false
//    property int heartRate
    property int rpm: 0
    property int level:10
    property bool showData: true
    property string message
    property bool dataHidingAuto: true
    property color overlayColor: "#444444"

    signal screenButtonPressed()

    function showMessage(message) {
        root.message = message;
        messageTimer.start();
    }

    function reset() {
        root.showData = true;
    }

    onVisibleChanged: {
        if(visible) {
            if(root.dataHidingAuto) {
                overlayTimer.restart()
            }

            root.showData = true
        }
    }

    MouseArea {
        anchors.fill: parent
        onClicked: {
            if(!root.showData && root.dataHidingAuto) {
                root.showData = true
                overlayTimer.restart()
            } else {
                screenButtonPressed()
            }

            buzzer.buttonPress()
        }
    }

    Rectangle {
        id: leftDisplay
        width: 150 * main.hScaleFactor
        height: 150 * main.vScaleFactor
        anchors.left: root.left
        anchors.verticalCenter: root.verticalCenter
        color: root.overlayColor
        opacity: 0.6
        visible: root.level > 0 || main.isTreadmill
        Column {
            anchors.fill: parent

            TextLabel {
                id: leftDataLabel
                width: parent.width
                height: parent.height / 4
                font.capitalization: Font.AllUppercase
                font.pixelSize: 18 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignBottom
                text: root.isTreadmill ? qsTr("INCLINE") : qsTr("LEVEL") + application.emptyString
            }

            NumberLabel {
                id: leftDataValue
                width: parent.width
                height: parent.height / 2
                font.capitalization: Font.AllUppercase
                font.pixelSize: 48 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                text: root.isTreadmill ? (root.incline/10.0).toFixed(1) : root.level
                onTextChanged: {
                    root.showData = true
                    overlayTimer.restart()
                }
            }

            TextLabel {
                id: leftDataUnits
                width: parent.width
                height: parent.height / 4
                font.capitalization: Font.AllUppercase
                font.pixelSize: 18 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignTop
                text: root.isTreadmill ? (qsTr("%") + application.emptyString) : ""
            }
        }
    }

    Rectangle {
        id: rightDisplay
        width: 150 * main.hScaleFactor
        height: 150 * main.vScaleFactor
        anchors.right: root.right
        anchors.verticalCenter: root.verticalCenter
        color: root.overlayColor
        opacity: 0.6

        Column {
            anchors.fill: parent

            TextLabel {
                id: rightDataLabel
                width: parent.width
                height: parent.height / 4
                font.capitalization: Font.AllUppercase
                font.pixelSize: 18 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignBottom
                text: ((root.isTreadmill || main.isStepmill) ? qsTr("SPEED") : qsTr("RPM")) + application.emptyString
            }
            
            NumberLabel {
                id: rightDataValue
                width: parent.width
                height: parent.height / 2
                font.capitalization: Font.AllUppercase
                font.pixelSize: 48 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                text: {
                       if (root.isTreadmill) return (root.speed/10.0).toFixed(1)
                       else if (main.isStepmill) return root.speed
                       else return root.rpm
                }
                onTextChanged: {
                    if ( root.isTreadmill ) {
                        root.showData = true
                        overlayTimer.restart()
                    }
                }
            }

            TextLabel {
                id: rightDataUnits
                width: parent.width
                height: parent.height / 4
                font.capitalization: Font.AllUppercase
                font.pixelSize: 18 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignTop
                text: (root.isTreadmill || main.isStepmill) ? (speedUnits + application.emptyString) : ""
            }
        }
    }

    Timer {
        id: overlayTimer
        interval: 10000
        repeat: false
        onTriggered: root.showData = false
    }

    Timer {
        id: messageTimer
        interval: 5000
        repeat: false
        running: false
    }

    state: root.showData ? "dataShown" : "dataHidden"
    states: [
        State {
            name: "dataShown"
            PropertyChanges { target: leftDisplay ; opacity: 0.6 }
            PropertyChanges { target: rightDisplay; opacity: 0.6 }
        },
        State {
            name: "dataHidden"
            PropertyChanges { target: leftDisplay ; opacity: 0 }
            PropertyChanges { target: rightDisplay; opacity: 0 }
        }
    ]
    transitions: [
        Transition {
            NumberAnimation { property: "opacity"; duration: 1000; easing.type: Easing.InOutQuad }
        }
    ]
}
