import QtQuick 2.12
//import VLCVideo 1.0
import WorkoutRunData 1.0
import "../string_format.js" as Format
import Components 1.0
import FileIO 1.0
import ".."

// Video window to play video
Rectangle {
    id: root
    color: "white"
    anchors.fill: parent

    property ListModel c360Modes: main.isLateralTrainer ? c360Modes_LateralTrainer : c360Modes_Elliptical
    property variant c360program: workoutProgram.c360Program
    property int phase: c360program.currentPhase
    property string videoPath: "/data/video/Cardio360/" + (main.isLateralTrainer ? "Traverse" : "Spectrum")
    property string videoSubpath: getVideoSubpath()
    property string introVideo: "Intro Clip.mp4"
    property int segmentTime: c360program.phaseTimeRemaining

    signal phaseTitleChanged(string text)
    signal playbackComplete()

    property bool isMenuDisplayed: false
    signal action()

    FileIO {
        id: fileio
    }

    function getVideoSubpath () {
        if(fileio.dirExists(videoPath + "/Female")) {
            return options.modestyMode ? "Male" : (Math.random() >= 0.5 ? "Female" : "Male")
        } else {
            return "Male"
        }
    }

    function beginWorkout() {
//        if (setVideoTimer.running) {
//            setVideoTimer.stop();
//        }

        root.phaseChanged(phase);
    }

//    Component.onCompleted: setVideoTimer.start() // Set video on delay to avoid race condition where video is not set
    Component.onCompleted: beginWorkout()

//    Timer {
//        id: setVideoTimer
//        interval: 300
//        running: false
//        onTriggered: {
//                beginWorkout()
//        }
//    }

    onPhaseChanged: {
        var text = qsTranslate("C360ViewContainer", c360Modes.title(phase))
        root.phaseTitleChanged(text)
        videoPlayer.mrl = "file://" + root.videoPath + "/" + root.videoSubpath + "/" + c360Modes.get(phase).video
        console.log("C360 video phase " + phase + " mrl: " + videoPlayer.mrl)
    }

    function stopVideo() {
        videoPlayer.stop();
    }

    C360Video {
        id: videoPlayer

        onMrlChanged: {
            console.log("C360 MRL changed to " + videoPlayer.mrl)
            videoPlayer.start()
        }
    }
/*
    VLCSurface {
        id: videoSurface
        backgroundColor: "white"
        player: videoPlayer
        aspect: videoPlayer.aspect
        playing: videoPlayer.playing
        anchors.fill: parent
    }

    VLCVideo {
        id: videoPlayer
        property bool fullscreen

        debug: false
        deinterlace: false
        loop: true
        onMrlChanged: {
            console.log("C360 MRL changed to " + videoPlayer.mrl)
        }
    }
*/
    ScalableImage {
        id: motionMan
        anchors {
            left: parent.left
            leftMargin: 40 * main.hScaleFactor
            top: parent.top
            topMargin: 85 * main.vScaleFactor
        }
        fillMode: Image.PreserveAspectFit
        smooth: true
        height: 260 * main.vScaleFactor
        width: 262 * main.hScaleFactor
        sourceName: "c360/MotionMan/" + c360Modes.get(phase).icon.replace(/\.[^/.]+$/, "");
    }

    Text {
        id: instText
        width: 300 * main.hScaleFactor
        anchors {
            right: parent.right
            rightMargin: 55 * main.hScaleFactor
            bottom: segRect.top
            bottomMargin: 20 * main.vScaleFactor
        }
        wrapMode: Text.WordWrap
        text: qsTranslate("C360ViewContainer", c360Modes.instructions(phase)) + application.emptyString
    }

    Item {
        id: segRect
        width: 270 * main.hScaleFactor
        height: childrenRect.height
        anchors {
            top: parent.top
            topMargin: 275 * main.vScaleFactor
            right: parent.right
            rightMargin: 96 * main.hScaleFactor
        }

        Text {
            id: segTimeLabel
            width: 270 * main.hScaleFactor
            horizontalAlignment: Text.AlignHCenter
            anchors.horizontalCenter: parent.horizontalCenter
            text: qsTranslate("C360View", "SEGMENT TIME REMAINING") + application.emptyString
            font.pointSize: 14 * main.vScaleFactor
            wrapMode: Text.Wrap
        }

        Text {
            id: segTimeIndicator
            anchors {
                top: segTimeLabel.bottom
                horizontalCenter: parent.horizontalCenter
            }
            text: Format.formatTimeString(root.segmentTime)
            font.pointSize: 45 * main.vScaleFactor
            font.bold: false
        }
    }
}
